#include "driver.h"
#include "wpc.h"
#include "wpcsim.h"
#include "wpcsound.h"

/* 161000 TAFG is not a clone of TAF anymore            */
/*        Added TAFG H3 ROM and Sound in TAFG           */
/*        Added Buy-in button and cleaned up BITIMP use */
/*        Added swBookOpen condition to Vault           */
/*        Adjusted timing. Thing Flip now works         */
/* 181000 Corrected year and names */
/* 231000 Now uses textOutf */
/*--------------
/ Keys
/   +R  L/R Ramp
/   +I  L/R Inlane
/   +-  L/R Slingshot
/   +L  L/R Loop
/   +O  L/R Outlane
/   V   Vault
/   S   Swamp
/   C   Chair
/   T   Thing Lane
/   Q   SDTM
/   B   Bookcase
/   I   Left Inner Inlane
/   H   Thing Hole
/   W   Train Wreck
/  EYU  G-R-A (V-E)
/ NM,./ Jet Bumpers
/ DFG   Swamp Milions
/  Z    Cousin It
/---------------------------*/
#define TAF_BOOKTICKS  200
#define TAF_THINGTICKS 400

/*------------------
/  Local functions
/-------------------*/
/*-- local state/ball handling functions --*/
static int taf_handleBallState(wpcsim_tBallStatus *ball, int *inports);
/*static int taf_getSol(int solNo);*/
static void taf_handleMech(int *inports);
static void taf_drawMech(unsigned char **line);
static void init_taf(void);

/*-----------------------
/ local static variables
/------------------------*/
static struct {
  int bookPos;  /* bookcase position */
  int thingPos; /* thing position */
} locals;

/*--------------------------
/ Game specific input ports
/---------------------------*/
WPC_INPUT_PORTS_START(taf,3)
  PORT_START /* 0 */
    WPCPORT_BIT(   0x0001,"Left Qualifier",  KEYCODE_LSHIFT)
    WPCPORT_BIT(   0x0002,"Right Qualifier", KEYCODE_RSHIFT)
    WPCPORT_BITIMP(0x0004,"L/R Ramp",        KEYCODE_R)
    WPCPORT_BITIMP(0x0008,"L/R Outlane",     KEYCODE_O)
    WPCPORT_BITIMP(0x0010,"L/R Loop",        KEYCODE_L)
    WPCPORT_BIT(   0x0040,"L/R Slingshot",   KEYCODE_MINUS)
    WPCPORT_BIT(   0x0080,"L/R Inlane",      KEYCODE_I)
    WPCPORT_BITIMP(0x0100,"Vault",           KEYCODE_V)
    WPCPORT_BITIMP(0x0200,"Swamp",           KEYCODE_S)
    WPCPORT_BITIMP(0x0400,"Chair",           KEYCODE_C)
    WPCPORT_BITIMP(0x0800,"Thing",           KEYCODE_T)
    WPCPORT_BIT(   0x1000,"Bookcase",        KEYCODE_B)
    WPCPORT_BIT(   0x2000,"Left Inner Inlane",KEYCODE_I)
    WPCPORT_BITIMP(0x4000,"Thing Hole",      KEYCODE_H)
    WPCPORT_BIT(   0x8000,"Train Wreck",     KEYCODE_W)
  PORT_START /* 1 */
    WPCPORT_BIT(   0x0001,"Grave",           KEYCODE_E)
    WPCPORT_BIT(   0x0002,"gRave",           KEYCODE_Y)
    WPCPORT_BIT(   0x0004,"grAve",           KEYCODE_U)
    WPCPORT_BIT(   0x0008,"It",              KEYCODE_Z)
    WPCPORT_BIT(   0x0010,"Upper Swamp",     KEYCODE_D)
    WPCPORT_BIT(   0x0020,"Center Swamp",    KEYCODE_F)
    WPCPORT_BIT(   0x0040,"Lower Swamp",     KEYCODE_G)
    WPCPORT_BIT(   0x0100,"Jet 1",           KEYCODE_N)
    WPCPORT_BIT(   0x0200,"Jet 2",           KEYCODE_M)
    WPCPORT_BIT(   0x0400,"Jet 3",           KEYCODE_COMMA)
    WPCPORT_BIT(   0x0800,"Jet 4",           KEYCODE_STOP)
    WPCPORT_BIT(   0x1000,"Jet 5",           KEYCODE_SLASH)
    WPCPORT_BITIMP(0x2000,"Drain",           KEYCODE_Q)
    WPCPORT_BIT(   0x4000,"Buy-in",          KEYCODE_2)
WPC_INPUT_PORTS_END

/*-------------------
/ Switch definitions
/--------------------*/
#define swBuyIn      11
#define swStart      13
#define swTilt       14
#define swLTrough    15
#define swCTrough    16
#define swRTrough    17
#define swOuthole    18

#define swSlamTilt   21
#define swCoinDoor   22
#define swTicket     23
#define swRIn        25
#define swROut       26
#define swShooter    27

#define swJet1       31
#define swJet2       32
#define swJet3       33
#define swJet4       34
#define swJet5       35
#define swLSling     36
#define swRSling     37
#define swLLoop      38

#define swGrave      41
#define swgRave      42
#define swChair      43
#define swIt         44
#define swLSwamp     45
#define swCSwamp     47
#define swUSwamp     48

#define swBallLane   51
#define swBook1      53
#define swBook2      54
#define swBook3      55
#define swBook4      56
#define swBumpLane   57
#define swRRampExit  58

#define swLRampEnt   61
#define swTrain      62
#define swThLane     63
#define swRRampEnt   64
#define swRRamp      65
#define swLRamp      66
#define swRLoop      67
#define swVault      68

#define swULock      71
#define swCLock      72
#define swLLock      73
#define swLKick      74
#define swLOut       75
#define swLIn2       76
#define swThKick     77
#define swLIn1       78

#define swBookOpen   81
#define swBookClose  82
#define swThingDn    84
#define swThingUp    85
#define swgrAve      86
#define swThHole     87

/*---------------------
/ Solenoid definitions
/----------------------*/
#define sChair        1
#define sThKnock      2
#define sRampDiv      3
#define sBallRel      4
#define sOuthole      5
#define sThMag        6
#define sThKick       7
#define sLKick        8
#define sJet1         9
#define sJet2        10
#define sJet3        11
#define sJet4        12
#define sJet5        13
#define sLSling      14
#define sRSling      15
#define sLMag        16
#define sUMag        23
#define sRMag        24
#define sThMotor     25
#define sThHole      26
#define sBookMotor   27
#define sSwampRel    28

/*---------------------
/  Ball state handling
/----------------------*/
enum {stRTrough=WPCSIM_FIRSTSTATE, stCTrough, stLTrough, stOuthole, stRIn,
      stROut, stShooter, stLLoopUp, stLLoopDn, stChair, stBallLane, stBumpLane,
      stRRampExit, stLRampEnt, stThLane, stRRampEnt, stRRamp, stLRamp,
      stRLoopUp, stRLoopDn, stVault, stULock, stCLock, stLLock, stLKick,
      stLOut, stThKick, stThHole,
      stDrain, stRampDiv, stTFlip, stUSwamp, stCSwamp, stLSwamp, stBallLane2,
      stThing};

static wpcsim_tState taf_stateDef[] = {
  {"Not Installed",    0,0,           0,        stDrain,     0,0,0,WPCSIM_STNOTEXCL},
  {"Moving"},
  {"PF",               0,0,           0,        0,           0,0,0,WPCSIM_STNOTEXCL},
  {"Right Trough",     1,swRTrough,   sBallRel, stShooter,   5},
  {"Center Trough",    1,swCTrough,   0,        stRTrough,   1},
  {"Left Trough",      1,swLTrough,   0,        stCTrough,   1},
  {"Outhole",          1,swOuthole,   sOuthole, stLTrough,   5},
  {"Right Inlane",     1,swRIn,       0,        stFree,      5},
  {"Right Outlane",    1,swROut,      0,        stDrain,    20},
  {"Ball Shooter",     1,swShooter,   sShooterRel,stBallLane, 10,0,0,WPCSIM_STNOTEXCL|WPCSIM_STSHOOT},
  {"Left Loop",        1,swLLoop,     0,        stRLoopDn,  20},
  {"Left Loop",        1,swLLoop,     0,        stFree,     10},
  {"Chair",            1,swChair,     sChair,   stFree,     10,0,0,WPCSIM_STNOTEXCL},
  {"Ball Lane",        1,swBallLane,  0,        0,           0,0,0,WPCSIM_STNOTEXCL},
  {"Bumper Lane",      1,swBumpLane},
  {"Right Ramp Exit",  1,swRRampExit, 0,        stRIn,       5},
  {"Left Ramp Enter",  1,swLRampEnt,  0,        stLRamp,     5},
  {"Thing Lane",       1,swThLane,    0,        stThHole,   10},
  {"Right Ramp Enter", 1,swRRampEnt,  0,        stRRamp,     5},
  {"Right Ramp",       1,swRRamp,     0,        stRampDiv,   5},
  {"Left Ramp",        1,swLRamp,     0,        stRampDiv,   5},
  {"Right Loop",       1,swRLoop,     0,        stLLoopDn,  10},
  {"Right Loop",       1,swRLoop,     0,        stFree,     10},
  {"Vault",            1,swVault,     0,        stULock,    20},
  {"Upper Lock",       1,swULock,     0,        stCLock,     1},
  {"Center Lock",      1,swCLock,     0,        stLLock,     1},
  {"Lower Lock",       1,swLLock,     sSwampRel,stLKick,     5},
  {"Lock Kickout",     1,swLKick,     sLKick,   stFree,     10},
  {"Left Outlane",     1,swLOut,      0,        stDrain,    20},
  {"Thing Kickout",    1,swThKick,    0,        stULock,    20},
  {"Thing Hole",       1,swThHole,    sThHole,  0,           0,0,0,WPCSIM_STIGNORESOL},
  {"Drain",            1,0,           0,        stOuthole,   0,0,0,WPCSIM_STNOTEXCL},
  {"Ramp Diverter",    1,0,           0,        stRRampExit,15,sRampDiv,stBumpLane},
  {"Thing flip",       1,0,           0,        0,           0},
  {"Swamp Million",    1,swUSwamp,    0,        stFree,     10},
  {"Swamp Million",    1,swCSwamp,    0,        stFree,     10},
  {"Swamp Million",    1,swLSwamp,    0,        stFree,     10},
  {"Upper Ball Lane",  1},
  {"On Thing",         1},
  {0}
};

static int taf_handleBallState(wpcsim_tBallStatus *ball, int *inports) {
  switch (ball->state) {
    case stBallLane:  if (ball->speed > 5)     return setState(stBallLane2,10);
                      else { ball->speed = -1; return setState(stULock,10); }
    case stBumpLane:  ball->custom = 0; return setState(stTFlip, 1);
    case stThHole:    if (wpc_getSw(swThingUp) && wpc_getSol(sThMag))
                        return setState(stThing,5);
                      if (wpc_getSol(sThHole))
                        return setFree(10);
                      break;
    case stTFlip:     if (wpc_getSol(sULFlip)) {
                        if (ball->custom < 1)  return setFree(5);
                        if (ball->custom < 2)  return setState(stUSwamp,10);
                        if (ball->custom < 3)  return setState(stULock, 15);
                        if (ball->custom < 4)  return setState(stCSwamp,10);
                        if (ball->custom < 5)  return setState(stLSwamp,10);
                      }
                      if (ball->custom >= 5)   return setFree(10);
                      ball->custom += 1;
                      break;
    case stBallLane2: if (ball->speed > 20) {
                        if (ball->speed < 25) return setState(stThHole,5);
                        else                  return setState(stRLoopDn,10);
                      }
                      else {ball->speed = -1; return setState(stBallLane,10); }
    case stThing:     if (!wpc_getSol(sThMag)) {
                        if (wpc_getSw(swThingUp)) return setState(stThHole, 1);
                        if (wpc_getSw(swThingDn)) return setState(stThKick,10);
                        /* ball released between switches ?? */
                      }
                      break;
  }
  return 0;
}

/*---------------------------
/  Keyboard conversion table
/----------------------------*/
static wpcsim_tInportData taf_inportData[] = {
  {0, 0x0005, stLRampEnt},
  {0, 0x0006, stRRampEnt},
  {0, 0x0009, stLOut},
  {0, 0x000a, stROut},
  {0, 0x0011, stLLoopUp},
  {0, 0x0012, stRLoopUp},
  {0, 0x0041, swLSling},
  {0, 0x0042, swRSling},
  {0, 0x0081, swLIn1},
  {0, 0x0082, stRIn},
  {0, 0x0100, stVault, swBookOpen},
  {0, 0x0200, stULock},
  {0, 0x0400, stChair},
  {0, 0x0800, stThLane},
  {0, 0x1000, swBook1},
  {0, 0x2000, swLIn2},
  {0, 0x4000, stThHole},
  {0, 0x8000, swTrain},
  {1, 0x0001, swGrave},
  {1, 0x0002, swgRave},
  {1, 0x0004, swgrAve},
  {1, 0x0008, swIt},
  {1, 0x0010, stUSwamp},
  {1, 0x0020, swCSwamp},
  {1, 0x0040, stLSwamp},
  {1, 0x0100, swJet1},
  {1, 0x0200, swJet2},
  {1, 0x0400, swJet3},
  {1, 0x0800, swJet4},
  {1, 0x1000, swJet5},
  {1, 0x2000, stDrain},
  {1, 0x4000, swBuyIn},
  {0}
};

/*--------------------
/ Drawing information
/---------------------*/
static void taf_drawMech(unsigned char **line) {
  wpc_textOutf(50, 0,BLACK,"Book: %-6s", wpc_getSw(swBookOpen) ? "Open" : (wpc_getSw(swBookClose) ? "Closed" : ""));
  wpc_textOutf(50,10,BLACK,"Thing: %-4s", wpc_getSw(swThingUp) ? "Up" : (wpc_getSw(swThingDn) ? "Down" : ""));
}

/*-----------------
/  ROM definitions
/------------------*/
WPC_ROMSTART(taf,l5,  "addam_l5.rom",0x80000,0x4c071564)
WPCS_SOUNDROM8("tafu18l1.rom",0x131ae471)
WPC_ROMEND

WPC_ROMSTART(tafg,lx3,"afgldlx3.rom",0x80000,0x0cc62fa5)
WPCS_SOUNDROM84("sndu18l1.rom",0x02e824a9,
                "sndu15l1.rom",0xb8c88c75)
WPC_ROMEND
WPC_ROMSTART(tafg,h3,"cpu-u6h3.rom",0x80000,0x2fe97098)
WPCS_SOUNDROM84("sndu18l1.rom",0x02e824a9,
                "sndu15l1.rom",0xb8c88c75)
WPC_ROMEND
/*--------------
/  Game drivers
/---------------*/
/*-- TAFG uses TAF input ports and init functions --*/
#define input_ports_tafg input_ports_taf
#define init_tafg        init_taf

WPC_GAMEDEF(taf,l5,      "TAF - The Addams Family L-5",       1992,"Bally",wpc_mFliptronS,0)
WPC_GAMEDEF(tafg,h3,     "TAFG - The Addams Family Special Collectors Edition H-3", 1994,"Bally",wpc_mFliptronS,0)
WPC_CLONEDEF(tafg,lx3,h3,"TAFG - The Addams Family Special Collectors Edition LX-3",1994,"Bally",wpc_mFliptronS,0)

/*----------
/ Game Data
/-----------*/
static wpcsim_tSimData tafSimData = {
  2,    /* 2 game specific input ports */
  taf_stateDef,
  taf_inportData,
  { stRTrough, stCTrough, stLTrough, stDrain, stDrain, stDrain, stDrain },
  NULL, /* no init */
  taf_handleBallState,
  taf_handleMech,
  TRUE, /* simulate manual shooter */
  NULL  /* no custom key conditions */
};

static wpc_tGameData tafGameData = {
  WPC_GENFLIPTRON, /* generation */
  {0},  /* no security chip */
  0,    /* no custom switch column */
  0,    /* no custom lampcolumns */
  0,    /* no custom solenoids/flashers */
  FLIP_BUT(FLIP_L) | FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x7c, 0x00, 0x00, 0x18, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0},
  NULL, /* no custom solenoid handler needed */
  NULL, NULL, NULL, /* switch, lamp & solenoid positions */
  taf_drawMech,
  &tafSimData
};


/*---------------
/  Game handling
/----------------*/
static void init_taf(void) {
  wpc_gameData = &tafGameData;
}

static void taf_handleMech(int *inports) {
  /*-- update counter if motors are running --*/
  if (wpc_getSol(sBookMotor)) locals.bookPos = (locals.bookPos + 1) % TAF_BOOKTICKS;
  if (wpc_getSol(sThMotor))   locals.thingPos = (locals.thingPos + 1) % TAF_THINGTICKS;
  /*-- update bookcase switches --*/
  wpc_setSw(swBookOpen, locals.bookPos < 15);
  wpc_setSw(swBookClose, (locals.bookPos >= (TAF_BOOKTICKS/2)) &&
                         (locals.bookPos <  (TAF_BOOKTICKS/2+15)));
  /*-- update thing switches --*/
  wpc_setSw(swThingDn, locals.thingPos < 15);
  wpc_setSw(swThingUp, (locals.thingPos >= (TAF_THINGTICKS/2)) &&
                       (locals.thingPos <  (TAF_THINGTICKS/2+15)));
}

