#include "driver.h"
#include "wpc.h"
#include "wpcsim.h"
#include "wpcsound.h"

/* 161000  Cleaned up BITIMP use */
/* 231000  Now uses textOutf */
#define TZ_GUMTIME      120 /* Time for Gumball machine to dispense a ball */
#define TZ_GUMRELTIME     3 /* Timer for gumball exit */
#define TZ_CLOCKTICKS    17 /* Time for clock to move 15 minutes */
#define TZ_POWERBALL   0x01

/*------------------
/  Local functions
/-------------------*/
static int tz_handleBallState(wpcsim_tBallStatus *ball, int *inports);
static int tz_getSol(int solNo);
static void tz_handleMech(int *inports);
static void tz_initSim(wpcsim_tBallStatus *balls, int *inports, int noOfBalls);
/*static void tz_drawCSol(struct osd_bitmap *bitmap, int fullRefresh, int y, int x, int color);*/
static void tz_drawMech(unsigned char **line);
static void init_tz(void);

/*-----------------------
/ local static variables
/------------------------*/
static struct tz_tLocals {
  int gumPos;     /* gumball motor position */
  int gumRel;     /* gumball release (faked solenoid) */
  int clockTick;  /* clock position (from noon) */
} locals;

/*--------------------------
/ Game specific input ports
/---------------------------*/
WPC_INPUT_PORTS_START(tz,6)
  PORT_START /* 0 */
    WPCPORT_BIT(   0x0001, "Left Qualifier",  KEYCODE_LSHIFT)
    WPCPORT_BIT(   0x0002, "Right Qualifier", KEYCODE_RSHIFT)
    WPCPORT_BITIMP(0x0004, "L/R Ramp",        KEYCODE_R)
    WPCPORT_BITIMP(0x0008, "L/R Outlane",     KEYCODE_O)
    WPCPORT_BITIMP(0x0010, "L/R Spiral",      KEYCODE_S)
    WPCPORT_BIT(   0x0020, "L/R Mini PF",     KEYCODE_M)
    WPCPORT_BIT(   0x0040, "L/R Slingshot",   KEYCODE_MINUS)
    WPCPORT_BIT(   0x0080, "L/R Inlane",      KEYCODE_I)
    WPCPORT_BIT(   0x0100, "Power Payoff",    KEYCODE_Y)
    WPCPORT_BIT(   0x0200, "Jet 1",           KEYCODE_V)
    WPCPORT_BIT(   0x0400, "Jet 2",           KEYCODE_B)
    WPCPORT_BIT(   0x0800, "Jet 3",           KEYCODE_N)
    WPCPORT_BITIMP(0x1000, "Mini PF Top",     KEYCODE_T)
    WPCPORT_BITIMP(0x2000, "Mini PF Exit",    KEYCODE_E)
    WPCPORT_BITIMP(0x4000, "Clock Lane",      KEYCODE_U)
    WPCPORT_BIT(   0x8000, "Buy-in",          KEYCODE_2)
  PORT_START /* 1 */
    WPCPORT_BIT(   0x0001, "Greed 1",         KEYCODE_A)
    WPCPORT_BIT(   0x0002, "Greed 2",         KEYCODE_F)
    WPCPORT_BIT(   0x0004, "Greed 3",         KEYCODE_G)
    WPCPORT_BIT(   0x0008, "Greed 4",         KEYCODE_J)
    WPCPORT_BIT(   0x0010, "Greed 5",         KEYCODE_K)
    WPCPORT_BIT(   0x0020, "Greed 6",         KEYCODE_COLON)
    WPCPORT_BIT(   0x0040, "Greed 7",         KEYCODE_QUOTE)
    WPCPORT_BIT(   0x0080, "Clock Target",    KEYCODE_W)
    WPCPORT_BIT(   0x0100, "Left Inner Inlane",KEYCODE_I)
    WPCPORT_BITIMP(0x0200, "Slot Machine",    KEYCODE_S)
    WPCPORT_BITIMP(0x0400, "Camera",          KEYCODE_C)
    WPCPORT_BITIMP(0x0800, "Piano",           KEYCODE_Z)
    WPCPORT_BITIMP(0x1000, "Dead End",        KEYCODE_D)
    WPCPORT_BITIMP(0x2000, "Lock",            KEYCODE_L)
    WPCPORT_BITIMP(0x4000, "Rocket",          KEYCODE_R)
    WPCPORT_BIT(   0x8000, "Hitchhiker",      KEYCODE_H)
  PORT_START /* 2 */
    WPCPORT_BITIMP(0x8000, "Drain",           KEYCODE_Q)
    WPCPORT_DIPNAME( 0x0007, 0x05, "Powerball")
      WPCPORT_DIPSET(0x0000, "1" )
      WPCPORT_DIPSET(0x0001, "2" )
      WPCPORT_DIPSET(0x0002, "3" )
      WPCPORT_DIPSET(0x0003, "4" )
      WPCPORT_DIPSET(0x0004, "5" )
      WPCPORT_DIPSET(0x0005, "6" )
      WPCPORT_DIPSET(0x0006, "7" )
    WPCPORT_DIPNAME( 0x0008, 0x00, "Third Magnet")
      WPCPORT_DIPSET(0x0000, DEF_STR(No) )
      WPCPORT_DIPSET(0x0008, DEF_STR(Yes) )
    WPCPORT_DIPNAME( 0x0010, 0x00, "Big Kick")
      WPCPORT_DIPSET(0x0000, DEF_STR(No) )
      WPCPORT_DIPSET(0x0010, DEF_STR(Yes) )
    WPCPORT_DIPNAME( 0x0020, 0x00, "Clock Lane")
      WPCPORT_DIPSET(0x0000, DEF_STR(No) )
      WPCPORT_DIPSET(0x0020, DEF_STR(Yes) )
WPC_INPUT_PORTS_END

/*-------------------
/ Switch definitions
/--------------------*/
#define swRIn         11
#define swROut        12
#define swStart       13
#define swTilt        14
#define swRTrough     15
#define swCTrough     16
#define swLTrough     17
#define swOuthole     18

#define swSlamTilt    21
#define swCoinDoor    22
#define swBuyIn       23
#define swFLTrough    25
#define swTProxi      26
#define swShooter     27
#define swRocket      28

#define swJet1        31
#define swJet2        32
#define swJet3        33
#define swLSling      34
#define swRSling      35
#define swLOut        36
#define swLIn1        37
#define swLIn2        38

#define swDeadEnd     41
#define swCamera      42
#define swPiano       43
#define swMPFEnter    44
#define swLMPF        45
#define swRMPF        46
#define swClockM      47
#define swGreed1      48

#define swGumLane     51
#define swHitchH      52
#define swLRampEnt    53
#define swLRamp       54
#define swGeneva      55
#define swGumExit     56
#define swSProxi      57
#define swSlot        58

#define swSkillR      61
#define swSkillO      62
#define swSkillY      63
#define swGreed4      64
#define swPowerPay    65
#define swGreed5      66
#define swGreed6      67
#define swGreed7      68

#define swBigKick     71
#define swAutoFire    72
#define swRRampEnt    73
#define swGumPop      74
#define swMPFTop      75
#define swMPFExit     76
#define swGreed2      77
#define swGreed3      78

#define swRLMag       81
#define swRUMag       82
#define swLMag        83
#define swCLock       84
#define swULock       85
#define swCLane       86
#define swGumEnter    87
#define swLLock       88

#define swClockM15   WPC_CUSTSWNO(1,1)
#define swClockM0    WPC_CUSTSWNO(1,2)
#define swClockM45   WPC_CUSTSWNO(1,3)
#define swClockM30   WPC_CUSTSWNO(1,4)
#define swClockH1    WPC_CUSTSWNO(1,5)
#define swClockH2    WPC_CUSTSWNO(1,6)
#define swClockH3    WPC_CUSTSWNO(1,7)
#define swClockH4    WPC_CUSTSWNO(1,8)

/*----------------------
/ Solenoid definitions
/----------------------*/
#define sSlot          1
#define sRocket        2
#define sAutoFire      3
#define sGumPop        4
#define sRRampDiv      5
#define sGumDiv        6
#define sKnocker       7
#define sOuthole       8
#define sBallRel       9
#define sRSling       10
#define sLSling       11
#define sJet1         12
#define sJet2         13
#define sJet3         14
#define sLockRel      15
#define sAutoDiv      16
#define sLMag         21
#define sRUMag        22
#define sRLMag        23
#define sGumMotor     24
#define sLMPFMag      25
#define sRMPFMag      26
#define sLRampDiv     27
#define sClockRev     WPC_CUSTSOLNO(5)
#define sClockFwd     WPC_CUSTSOLNO(6)
/*-- fake solenoids to simplify the state handling--*/
#define sGumRel       WPC_CUSTSOLNO(7)

/*---------------------
/  Ball state handling
/----------------------*/
enum {stMPFFree=WPCSIM_FIRSTSTATE, stRTrough, stCTrough, stLTrough, stFLTrough, stShooterDiv,
      stLLock, stCLock, stULock, stRLMagDn, stRLMagUp, stRUMagDn, stRUMagUp, stLMagDn,
      stLMagUp, stSpiral, stGumLane, stGumPop, stGumEntry, stGum4, stGum3, stGum2,
      stGum1, stGumExit, stCamera, stPiano, stDeadEnd, stSProx, stSlot, stLRampEnt,
      stLRamp, stRRampEnt, stRRamp, stRRampDiv, stMPFEnter, stMPFTop, stMPFExit,
      stDrain, stOuthole, stRocket, stAutoFire, stShooter, stBallLane, stSkillR, stSkillO,
      stSkillY, stLIn2, stHitchH, stLOut, stROut, stBigKick};

static wpcsim_tState tz_stateDef[] = {
 /* Display         swTime switch     sol       next    timer alt flags*/
  {"Not Installed",      0,0,         0,        stDrain,    0,0,0,WPCSIM_STNOTEXCL},
  {"Moving"},
  {"PF",                 0,0,         0,        0,          0,0,0,WPCSIM_STNOTEXCL},
  {"Mini PF",            0,0,         0,        0,          0,0,0,WPCSIM_STNOTEXCL},
  {"Right Trough",       5,swRTrough, sBallRel, 0,          0,0,0,WPCSIM_STIGNORESOL},
  {"Center Trough",      1,swCTrough, 0,        stRTrough,  1},
  {"Left Trough",        1,swLTrough, 0,        stCTrough,  1},
  {"Far Left Trough",    1,swFLTrough,0,        stLTrough,  1},
  {"Auto Diverter",      1,0,         0,        stShooter,  2,sAutoDiv,stAutoFire},
  {"Lower Lock",         1,swLLock,   sLockRel, stRLMagDn, 10},
  {"Center Lock",        1,swCLock,   0,        stLLock,    1},
  {"Upper Lock",         1,swULock,   0,        stCLock,    1},
  {"Lower Right Magnet", 1,swRLMag,   0,        0,          0,0,0,WPCSIM_STNOTEXCL},
  {"Lower Right Magnet", 1,swRLMag,   0,        0,          0,0,0,WPCSIM_STNOTEXCL},
  {"Upper Right Magnet", 1,swRUMag,   0,        0,          0,0,0,WPCSIM_STNOTEXCL},
  {"Upper Right Magnet", 1,swRUMag,   0,        0,          0,0,0,WPCSIM_STNOTEXCL},
  {"Left Magnet",        1,swLMag,    0,        0,          0,0,0,WPCSIM_STNOTEXCL},
  {"Left Magnet",        1,swLMag,    0,        0,          0,0,0,WPCSIM_STNOTEXCL},
  {"In Spiral",          1,0,         0,        stLMagDn,   5,sGumDiv,stGumLane},
  {"Gumball Lane",       1,swGumLane, 0,        stGumPop,  20},
  {"Gumball Popper",     1,swGumPop,  sGumPop,  stGumEntry, 3},
  {"Gumball Entry",      5,swGumEnter,0,        stGum4,     1},
  {"Gumball 4",          1,0,         0,        stGum3,     1},
  {"Gumball 3",          1,0,         0,        stGum2,     1},
  {"Gumball 2",          1,0,         0,        stGum1,     1},
  {"Gumball 1",          1,0,         sGumRel,  stGumExit, 15},
  {"Gumball Exit",       1,swGumExit, 0,        stCamera,  15},
  {"Camera",             1,swCamera,  0,        stSProx,   10},
  {"Piano",              1,swPiano,   0,        stSProx,   10},
  {"DeadEnd",            1,swDeadEnd, 0,        stCamera,   5},
  {"Slot Prox",          5,swSProxi,  0,        stSlot,     5,0,0,WPCSIM_STNOTEXCL|WPCSIM_STNOTBALL(TZ_POWERBALL)},
  {"Slot Machine",       1,swSlot,    sSlot,    stFree,     5,0,0,WPCSIM_STNOTEXCL},
  {"Left Ramp Enter",    1,swLRampEnt,0,        stLRamp,   10},
  {"Left Ramp",          1,swLRamp,   0,        stLIn2,    15,sLRampDiv,stShooterDiv},
  {"Right Ramp Enter",   1,swRRampEnt,0,        stRRamp,    4},
  {"Right Ramp Rail",    0,0,         0,        stRRampDiv, 1,sRRampDiv,stMPFEnter},
  {"Right Ramp Diverter",0,0,         sRRampDiv,stFree,    10},
  {"Mini PF Enter",      1,swMPFEnter,0,        stMPFFree,  5},
  {"Mini PF Top",        1,swMPFTop,  0,        stCamera,  10},
  {"Mini PF Exit",       1,swMPFExit, 0,        stFree,    10},
  {"Drain",              0,0,         0,        stOuthole,  1,0,0, WPCSIM_STNOTEXCL},
  {"Outhole",            1,swOuthole, sOuthole, stFLTrough, 3},
  {"Rocket Kicker",      1,swRocket,  sRocket,  stHitchH,   5},
  {"Auto Fire",          1,swAutoFire,sAutoFire,stRLMagUp, 10,0,0, WPCSIM_STNOTEXCL},
  {"Shooter",            1,swShooter, sShooterRel,stBallLane, 5,0,0, WPCSIM_STNOTEXCL|WPCSIM_STSHOOT},
  {"Ball Lane",          0,0,         0,        0,          0,0,0, WPCSIM_STNOTEXCL},
  {"Red Skill"},
  {"Orange Skill",       1,swSkillO},
  {"Yellow Skill",       1,swSkillY},
  {"Left Inlane2",       1,swLIn2,    0,        stFree,     5},
  {"Hitchhiker",         1,swHitchH,  0,        stFree,    10},
  {"Left Outlane",       1,swLOut,    0,        stDrain,   10},
  {"Right Outlane",      1,swROut,    0,        stDrain,   10},
  {"Big Kick",           1,swBigKick}
};

#define setMPFFree(delay) setState(stMPFFree, delay)
static int tz_handleBallState(wpcsim_tBallStatus *ball, int *inports) {
  switch (ball->state) {
    case stRTrough:  if (wpc_getSol(sBallRel)) { wpc_setSw(swTProxi, FALSE); return setState(stShooterDiv, 6); }
                     else                      { wpc_setSw(swTProxi, !(ball->type & TZ_POWERBALL)); break; }
    case stRLMagDn:  if ((ball->type & TZ_POWERBALL) || !wpc_getSol(sRLMag)) return setFree(10);
                     else                                                    break;
    case stRLMagUp:  if ((ball->type & TZ_POWERBALL) || !wpc_getSol(sRLMag)) {
                       if (inports[2] & 0x0008) return setState(stRUMagUp, 5);
                       else                     return setState(stSpiral, 10);
                     }
                     else { forceState(stRLMagDn); break; } /* change direction */
    case stRUMagDn:  if ((ball->type & TZ_POWERBALL) || !wpc_getSol(sRUMag)) return setState(stRLMagDn,10);
                     else                                                    break;
    case stRUMagUp:  if ((ball->type & TZ_POWERBALL) || !wpc_getSol(sRUMag)) return setState(stSpiral, 5);
                     else { forceState(stRLMagDn); break; } /* change direction */
    case stLMagDn:   if ((ball->type & TZ_POWERBALL) || !wpc_getSol(sLMag)) return setFree(10);
                     else                                                   break;
    case stLMagUp:   if ((ball->type & TZ_POWERBALL) || !wpc_getSol(sLMag)) {
                       if (inports[2] & 0x0008) return setState(stRUMagDn, 10);
                       else                     return setState(stRLMagDn, 10);
                     }
                     else { forceState(stLMagDn); break; } /* change direction */
    case stBallLane: if (ball->speed > 10) return setState(stSkillR,5);
                     else                  return setState(stShooter, 5);
    case stSkillR:   if ((ball->speed > 13) && !wpc_getSw(swSkillO)) return setState(stSkillO, 1);
                     else { ball->speed = -1; return setState(stRocket, 20); }
    case stSkillO:   if ((ball->speed > 17) && !wpc_getSw(swSkillY)) return setState(stSkillY, 1);
                     else { ball->speed = -1; return setState(stSkillR, 2); }
    case stSkillY:   if (ball->speed > 20) { ball->speed = -1; return setState(stSlot, 5); }
                     else { ball->speed = -1; return setState(stSkillO, 2); }
                     break;
    case stBigKick:  if (wpc_getSol(sAutoFire)) return setState(stRLMagUp, 10);
                     else if (!wpc_getSw(swAutoFire)) return setState(stAutoFire, 2);
                     break;
  }
  return 0;
}

static void tz_initSim(wpcsim_tBallStatus *balls, int *inports, int noOfBalls) {

  initBallType(inports[2] & 0x0007, TZ_POWERBALL);

  /*-- set hardware status --*/
  wpc_updInvSw(swRUMag,   (inports[2] & 0x0008));
  wpc_updInvSw(swBigKick, (inports[2] & 0x0010));
  wpc_updInvSw(swCLane,   (inports[2] & 0x0020));
}

/*---------------------------
/  Keyboard conversion table
/----------------------------*/
static wpcsim_tInportData tz_inportData[] = {
  {0, 0x0005, stLRampEnt},
  {0, 0x0006, stRRampEnt},
  {0, 0x0011, stLMagUp},
  {0, 0x0012, stRLMagUp},
  {0, 0x0009, stLOut},
  {0, 0x000a, stROut},
  {0, 0x0021, swLMPF,   stMPFFree},
  {0, 0x0022, swRMPF,   stMPFFree},
  {0, 0x0041, swLSling},
  {0, 0x0042, swRSling},
  {0, 0x0081, swLIn1},
  {0, 0x0082, swRIn},
  {0, 0x0100, swPowerPay},
  {0, 0x0200, swJet1},
  {0, 0x0400, swJet2},
  {0, 0x0800, swJet3},
  {0, 0x1000, stMPFTop,  stMPFFree},
  {0, 0x2000, stMPFExit, stMPFFree},
  {0, 0x4000, swCLane},
  {0, 0x8000, swBuyIn, WPCSIM_STANY},
  {1, 0x0001, swGreed1},
  {1, 0x0002, swGreed2},
  {1, 0x0004, swGreed3},
  {1, 0x0008, swGreed4},
  {1, 0x0010, swGreed5},
  {1, 0x0020, swGreed6},
  {1, 0x0040, swGreed7},
  {1, 0x0080, swClockM},
  {1, 0x0100, swLIn2},
  {1, 0x0200, stSlot},
  {1, 0x0400, stCamera},
  {1, 0x0800, stPiano},
  {1, 0x1000, stDeadEnd},
  {1, 0x2000, stULock},
  {1, 0x4000, stRocket},
  {1, 0x8000, stHitchH},
  {2, 0x8000, stDrain, WPCSIM_STANY},
  {0}
};

/*--------------------
/ Drawing information
/---------------------*/
static wpc_tDrawData tz_lampPos[] = {
{ 0, 0 }, /* top left */
{36, 27}, /* size */
/* 11 */  {28,13,ORANGE}, /* 12 */  {28,11,ORANGE}, /* 13 */  {26,10,ORANGE}, /* 14 */  {24,10,ORANGE},
/* 15 */  {22,10,ORANGE}, /* 16 */  {20,10,ORANGE}, /* 17 */  {18,11,ORANGE}, /* 18 */  {18,13,ORANGE},
/* 21 */  {25,14,RED},    /* 22 */  {28,15,ORANGE}, /* 23 */  {26,16,ORANGE}, /* 24 */  {24,16,ORANGE},
/* 25 */  {22,16,ORANGE}, /* 26 */  {20,16,ORANGE}, /* 27 */  {18,15,ORANGE}, /* 28 */  {21,14,RED},
/* 31 */  {27, 1,RED},    /* 32 */  {25,12,RED},    /* 33 */  {26, 3,WHITE},  /* 34 */  {23,12,WHITE},
/* 35 */  {26, 5,WHITE},  /* 36 */  {21,12,RED},    /* 37 */  {24, 5,YELLOW}, /* 38 */  {20, 6,WHITE},
/* 41 */  {32, 8,YELLOW}, /* 42 */  {31,10,YELLOW}, /* 43 */  {30,12,YELLOW}, /* 44 */  {30,14,YELLOW},
/* 45 */  {31,16,YELLOW}, /* 46 */  {32,18,RED},    /* 47 */  {34,13,RED},    /* 48 */  {26,21,WHITE},
/* 51 */  { 5,13,WHITE},  /* 52 */  { 7,13,WHITE},  /* 53 */  { 9,13,ORANGE}, /* 54 */  { 6,11,WHITE},
/* 55 */  {14,11,WHITE},  /* 56 */  { 6,16,ORANGE}, /* 57 */  { 3,19,RED},    /* 58 */  { 1,19,ORANGE},
/* 61 */  {18, 2,RED},    /* 62 */  {22, 5,RED},    /* 63 */  {17, 7,RED},    /* 64 */  {20, 8,YELLOW},
/* 65 */  {17, 9,YELLOW}, /* 66 */  {27,23,RED},    /* 67 */  {14,21,WHITE},  /* 68 */  {12,23,ORANGE},
/* 71 */  {16,17,YELLOW}, /* 72 */  {14,16,YELLOW}, /* 73 */  {12,16,YELLOW}, /* 74 */  {10,17,WHITE},
/* 75 */  { 5,19,YELLOW}, /* 76 */  {11, 6,WHITE},  /* 77 */  {11, 2}, /* 78 */  { 9, 4},
/* 81 */  { 8,10,ORANGE}, /* 82 */  { 5,15,WHITE},  /* 83 */  { 7,21,YELLOW}, /* 84 */  { 9,21,RED},
/* 85 */  {14,19,YELLOW}, /* 86 */  {15,23,RED},    /* 87 */  {36,21,ORANGE}, /* 88 */  {36, 3,RED}
};

static void tz_drawMech(unsigned char **line) {
  wpc_textOutf(50, 0,BLACK,"%2d:%02d",locals.clockTick/(4*TZ_CLOCKTICKS),
                          (locals.clockTick*15/TZ_CLOCKTICKS) % 60);
  wpc_textOutf(50,10,BLACK,"Gum:%3d",locals.gumPos);
  wpc_textOutf(30, 40,BLACK,"Board 1 %4x", wpc_data[WPC_EXTBOARD1]);
}

/*-----------------
/  ROM definitions
/------------------*/
WPC_ROMSTART(tz, 94h, "tz_94h.rom", 0x80000, 0x5032e8c6)
WPCS_SOUNDROM882("tzu18_l2.rom", 0x66575ec2,
                 "tzu15_l2.rom", 0x389d2442,
                 "tzu14_l2.rom", 0x5a67bd56)
WPC_ROMEND

WPC_ROMSTART(tz,92,"tzone9_2.rom",0x80000,0xec3e61c8)
WPCS_SOUNDROM882("tzu18_l2.rom",0x66575ec2,
                 "tzu15_l2.rom",0x389d2442,
                 "tzu14_l2.rom",0x5a67bd56)
WPC_ROMEND

/*--------------
/  Game drivers
/---------------*/
WPC_GAMEDEF (tz,94h,     "TZ - Twilight Zone 9.4H",1993,"Bally",wpc_mFliptronS,0)
WPC_CLONEDEF(tz,92, 94h, "TZ - Twilight Zone 9.2", 1993,"Bally",wpc_mFliptronS,0)

/*----------
/ Game Data
/-----------*/
static wpcsim_tSimData tzSimData = {
  3,        /* 5 game specific input ports */
  tz_stateDef,
  tz_inportData,
  { stRTrough, stCTrough, stLTrough, stGum1, stGum2, stGum3, stFLTrough },
  tz_initSim,
  tz_handleBallState,
  tz_handleMech,
  TRUE, /* simulate manual Shooter */
  NULL, /* no custom conditions */
};

static wpc_tGameData tzGameData = {
  WPC_GENFLIPTRON, /* generation */
  {0},  /* no security chip */
  1,    /* 1 custom switch column */
  0,    /* no custom lampcolumns */
  6,    /* 6 custom solenoids/flashers */
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_U),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3f, 0x7f, 0x00, 0x00, 0x00, 0xff}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0},
  tz_getSol,
  NULL, tz_lampPos, NULL, /* switch, lamp and solenoid positions */
  tz_drawMech,
  &tzSimData
};

/*---------------
/  Game handling
/----------------*/
/*-- patched memory read function to handle the 9th switch column --*/
READ_HANDLER(tz_swRowRead) {
  if ((wpc_data[WPC_EXTBOARD1] & 0x80) > 0) /* 9th column enabled */
    return wpc_globals.swMatrix[WPC_CUSTSWCOL];
  return wpc_r(WPC_SWROWREAD);
}

static void init_tz(void) {
  wpc_gameData = &tzGameData;
  install_mem_read_handler(WPC_CPUNO, WPC_SWROWREAD+WPC_BASE, WPC_SWROWREAD+WPC_BASE,
                           tz_swRowRead);
}

static int tz_getSol(int solNo) {
  if (solNo == sClockRev)
    return (wpc_data[WPC_EXTBOARD1] & 0x40) > 0;
  if (solNo == sClockFwd)
    return (wpc_data[WPC_EXTBOARD1] & 0x20) > 0;
  if (solNo == sGumRel)
    return locals.gumRel > 0;
  return 0;
}

static void tz_handleMech(int *inports) {
  /*---------------
  /  handle clock
  /----------------*/
  switch (2*wpc_getSol(sClockFwd)+wpc_getSol(sClockRev)) {
    case 0:
    case 3: break;
    case 1: locals.clockTick += 1; if (locals.clockTick == TZ_CLOCKTICKS*48) locals.clockTick = 0; break;
    case 2: locals.clockTick -= 1; if (locals.clockTick == -1) locals.clockTick = TZ_CLOCKTICKS*48-1; break;
  }
  { /* convert clockTick to optos */
    static int hour2sw[12] = {0x09,0x01,0x00,0x04,0x0c,0x0d,0x05,0x07,0x06,0x02,0x03,0x0b};
    int tickM = locals.clockTick % (TZ_CLOCKTICKS * 4);
    int hour = hour2sw[((locals.clockTick+2*TZ_CLOCKTICKS)/(4*TZ_CLOCKTICKS)+12) % 12];

    wpc_setSw(swClockM15, (tickM == TZ_CLOCKTICKS));
    wpc_setSw(swClockM0,  (tickM == 0));
    wpc_setSw(swClockM30, (tickM == TZ_CLOCKTICKS*2));
    wpc_setSw(swClockM45, (tickM == TZ_CLOCKTICKS*3));
    wpc_setSw(swClockH4, hour & 0x08);
    wpc_setSw(swClockH3, hour & 0x04);
    wpc_setSw(swClockH2, hour & 0x02);
    wpc_setSw(swClockH1, hour & 0x01);
  }
  /*-------------------------
  /  handle gumball machine
  /--------------------------*/
  if (locals.gumRel > 0)
    locals.gumRel -= 1;
  if (wpc_getSol(sGumMotor)) {
    locals.gumPos = (locals.gumPos + 1) % TZ_GUMTIME;
    if (locals.gumPos == 0)
      locals.gumRel = TZ_GUMRELTIME;
  }
  /* dont' know how much "slack" there are in the geneva switch */
  wpc_setSw(swGeneva, locals.gumPos < TZ_GUMTIME/5); /* 20% slack */
}

