#ifndef INC_WPC
#define INC_WPC

/* 221000  Added wpc_textOutf */
#ifndef FALSE
  #define FALSE (0)
#endif
#ifndef TRUE
  #define TRUE (1)
#endif

#ifdef MAME_DEBUG
  #define DBGLOG(x) logerror x
#else /* MAME_DEBUG */
  #define DBGLOG(x)
#endif /* MAME_DEBUG */

/*-- my PC runs Windows NT and I can't get DOS sound to work --*/
/*-- this allows me to do some sound CPU debugging anyway    --*/
#if defined(MAME_DEBUG) && defined(WIN_NT)
  #define FORCESOUNDCPU 1
  #define REG_SOUNDONLY 0
  #define CPU_SOUNDONLY 0
#else
  #define FORCESOUNDCPU 0
  #define REG_SOUNDONLY REGIONFLAG_SOUNDONLY
  #define CPU_SOUNDONLY CPU_AUDIO_CPU
#endif

/*-- Standard input ports --*/
#define WPC_INPUT_PORTS_START(name,balls) \
  INPUT_PORTS_START(name) \
    WPC_PORTS \
    WPCSIM_PORTS(balls)
#define WPC_INPUT_PORTS_END INPUT_PORTS_END

/*-- some convenience macros --*/
/* strange but there are no way to define IMP and TOG with key without using BITX */
#define WPCPORT_BIT(mask, name, key) \
   PORT_BITX(mask,IP_ACTIVE_HIGH,IPT_BUTTON1,name,key,IP_JOY_NONE)
#define WPCPORT_BITIMP(mask, name, key) \
   PORT_BITX(mask,IP_ACTIVE_HIGH,IPT_BUTTON1 | IPF_IMPULSE | (1<<8),name,key,IP_JOY_NONE)
#define WPCPORT_BITTOG(mask, name, key) \
   PORT_BITX(mask,IP_ACTIVE_HIGH,IPT_BUTTON1 | IPF_TOGGLE,name,key,IP_JOY_NONE)
#define WPCPORT_DIPNAME(mask,default,name) \
   PORT_DIPNAME(mask,default,name)
#define WPCPORT_DIPSET(mask,name) \
   PORT_DIPSETTING(mask,name)

/*-- only used in standard inport --*/
#define WPCPORT_BITDEF(mask, type, key) \
   PORT_BITX(mask,IP_ACTIVE_HIGH, type, IP_NAME_DEFAULT, key, IP_JOY_DEFAULT)

/*-- WPC generations --*/
#define WPC_GENALPHA    0  /* Alpha-numeric display, Dr Dude 10/90 - The Machine BOP 4/91*/
#define WPC_GENDMD      1  /* Dot Matrix Display, Terminator 2 7/91 - Party Zone 10/91 */
#define WPC_GENFLIPTRON 2  /* Fliptronic flippers, Addams Family 2/92 - Twilight Zone 5/93 */
#define WPC_GENDCS	3  /* DCS Sound system, Indiana Jones 10/93 - Popeye 3/94 */
#define WPC_GENSECURITY 4  /* Security chip, World Cup Soccer 3/94 - Jackbot 10/95 */
#define WPC_GEN95       5  /* Integreted boards, Congo 3/96 - Cactus Canyon 2/99 */

/*-------------------------
/ Machine driver constants
/--------------------------*/
#define WPC_CPUNO  0
#define WPC_SCPUNO 1

/*-- Memory regions --*/
#define WPC_MEMREG_CPU		REGION_CPU1
#define WPC_MEMREG_SCPU		REGION_CPU2
#define	WPC_MEMREG_DMD		REGION_USER1
#define WPC_MEMREG_ROM		REGION_USER2
#define WPC_MEMREG_SROM		REGION_SOUND1
#define WPC_MEMREG_SBANK        REGION_USER3

/*-- Main CPU regions and ROM --*/
#define WPC_ROMSTART(name, ver, n1, size, chk1) \
   ROM_START(name##_##ver) \
     ROM_REGION(0x10000, WPC_MEMREG_CPU) \
     ROM_REGION(0x2000,  WPC_MEMREG_DMD) \
     ROM_REGION(size,    WPC_MEMREG_ROM) \
       ROM_LOAD(n1, 0x00000, size, chk1)
#define WPC_ROMEND ROM_END

/*-- Colours --*/
#define BLACK       0
#define WHITE       11
#define GREEN       12
#define RED         14
#define ORANGE      15
#define YELLOW      16
#define SHADE(x)    (x-6)

/*----------------------------------
/  Start address for the WPC chip
/-----------------------------------*/
#define WPC_BASE        0x3fb0

/*----------------------------------
/ The WPC registers I know about
/ Some registers were moved in later
/ WPC generations. These end with "95"
/-----------------------------------*/

/*----------------------------------
/  WPC_FLIPPERS write (active low)
/   0  LL Stroke    4 LU Stroke
/   1  LL Hold      5 LU Hold
/   2  RL Stroke    6 RU Stroke
/   3  RL Hold      7 RU Hold
/-----------------------------------*/
/*-- WPC bit shifter -------------------------------------------------------
/ This function adds on a bit level
/ SHIFTADRH and SHIFTADRL forms a 16 bit base address
/ SHIFTBIT is the number of bits to add
/ Writing to registers only stores the value
/ When the address is read the value of SHIFTBIT is added to the stored address
/ Reading the SHIFTBIT returns the bit position
/ Write address 0x1000 and SHIFTBIT 0x26. The values read will then be:
/ address=0x1004 and SHIFTBIT=0x10
/ or in c-syntax:
/   address = address + SHIFTBIT/8
/   SHIFTBIT = 1<<(SHIFTBIT & 7)
/
/ SHIFTBIT2 works in the same way but the address is unaffected
/   SHIFTBIT2 = 1<<(SHIFTBIT2 & 7)
/
/ Dr. Who is the only game testing the shifter.
/------------------------------------------------------------------------------
/  					      Generations
/------------------------------------------------AMFDS9-------------------------*/
#define WPC_PRINTBUSY     (0x3fc0 - WPC_BASE) /* xxxxx  R: Printer ready ??? */
#define WPC_PRINTDATA     (0x3fc1 - WPC_BASE) /* xxxxx  W: send to printer */
#define WPC_PRINTDATAX    (0x3fc2 - WPC_BASE) /* xxxxx  W: 0: Printer data available */
#define WPC_FLIPPERS      (0x3fd4 - WPC_BASE) /*   xxx  R: switches W: Solenoids */
#define WPC_SOUNDIF       (0x3fdc - WPC_BASE) /* xxx    RW: Sound board interface */
#define WPC_SOUNDBACK	  (0x3fdd - WPC_BASE) /* xxx    RW: R: Sound data availble, W: Big speaker ? */
#define WPC_SOLENOID1     (0x3fe0 - WPC_BASE) /* xxxxxx W: Solenoid 25-28 */
#define WPC_SOLENOID2     (0x3fe1 - WPC_BASE) /* xxxxxx W: Solenoid  1- 8 */
#define WPC_SOLENOID3     (0x3fe2 - WPC_BASE) /* xxxxxx W: Solenoid 17-24 */
#define WPC_SOLENOID4     (0x3fe3 - WPC_BASE) /* xxxxxx W: Solenoid  9-16 */
#define WPC_LAMPROW       (0x3fe4 - WPC_BASE) /* xxxxxx W: Lamp row */
#define WPC_LAMPCOLUMN    (0x3fe5 - WPC_BASE) /* xxxxxx W: Lamp column enable */
#define WPC_GILAMPS       (0x3fe6 - WPC_BASE) /*        W: GI lights ?? */
#define WPC_DIPSWITCH     (0x3fe7 - WPC_BASE) /* xxxxxx R: CPU board dip-switches */
#define WPC_SWCOINDOOR    (0x3fe8 - WPC_BASE) /* xxxxxx W: Coin door switches */
#define	WPC_SWROWREAD	  (0x3fe9 - WPC_BASE) /* xxxx   R: Switch row read */
#define WPC_PICREAD	  (0x3fe9 - WPC_BASE) /*     xx R: PIC data */
#define	WPC_SWCOLSELECT	  (0x3fea - WPC_BASE) /* xxxx   W: Switch column enable */
#define WPC_PICWRITE	  (0x3fea - WPC_BASE) /*     xx R: PIC data */
#define WPC_EXTBOARD1     (0x3feb - WPC_BASE) /*   x    W: Extension Driver Board 1 */
#define WPC_ALPHAPOS      (0x3feb - WPC_BASE) /* x      W: Select alphanumeric position */
#define WPC_EXTBOARD2	  (0x3fec - WPC_BASE) /*   x    W: Extension Driver Board 2 */
#define WPC_ALPHA1HI      (0x3fec - WPC_BASE) /* x      W: Display 1st row hi bits */
#define WPC_EXTBOARD3     (0x3fed - WPC_BASE) /*   x    W: Extension Driver Board 3 */
#define WPC_ALPHA1LO      (0x3fed - WPC_BASE) /* x      W: Display 1st row lo bits */
#define WPC_EXTBOARD4     (0x3fee - WPC_BASE) /*   x    W: Extension Driver Board 4 */
#define WPC_FLIPPERCOIL95 (0x3fee - WPC_BASE) /*      x W: Flipper Solenoids */
#define WPC_ALPHA2HI      (0x3fee - WPC_BASE) /* x      W: Display 2nd row hi bits */
#define WPC_EXTBOARD5	  (0x3fef - WPC_BASE) /*   x    W: Extension Driver Board 5 */
#define WPC_FLIPPERSW95   (0x3fef - WPC_BASE) /*      x R: Flipper switches */
#define WPC_ALPHA2LO      (0x3fee - WPC_BASE) /* x      W: Display 2nd row lo bits */
#define WPC_LED		  (0x3ff2 - WPC_BASE) /* xxxxxx W: CPU LED (bit 7) */
#define WPC_IRQACK        (0x3ff3 - WPC_BASE) /*        W: IRQ Ack ??? */
#define WPC_SHIFTADRH     (0x3ff4 - WPC_BASE) /* xxxxxx RW: See above */
#define WPC_SHIFTADRL	  (0x3ff5 - WPC_BASE) /* xxxxxx RW: See above */
#define WPC_SHIFTBIT	  (0x3ff6 - WPC_BASE) /* xxxxxx RW: See above */
#define WPC_SHIFTBIT2     (0x3ff7 - WPC_BASE) /* xxxxxx RW: See above */
#define WPC_FIRQSRC       (0x3ff8 - WPC_BASE) /*   xxxx R: bit 7 0=DMD, 1=SOUND? W: Clear FIRQ line */
#define WPC_RTCHOUR	  (0x3ffa - WPC_BASE) /* xxxxxx RW: Real time clock: hour */
#define WPC_RTCMIN	  (0x3ffb - WPC_BASE) /* xxxxxx RW: Real time clock: minute */
#define WPC_RTCLOAD	  (0x3ffe - WPC_BASE) /* xxxxxx W: Load new values to RTC */
#define WPC_ROMBANK	  (0x3ffc - WPC_BASE) /* xxxxxx W: Rombank switch */
#define WPC_WATCHDOG      (0x3fff - WPC_BASE) /* xxxxxx W: Watchdog */

/*----------------------
/ WPC driver constants
/-----------------------*/
#define WPC_MAXSWCOL      16  /* switch columns (0-9=sw matrix, 10=coin door, 11=cabinet/flippers) */
#define WPC_COINDOORSWCOL 0   /* internal array number */
#define WPC_CABINETSWCOL  11  /* internal array number */
#define WPC_CUSTSWCOL     12  /* first custom (game specific) switch column */
#define WPC_MAXLAMPCOL    16  /* lamp column (0-7=std lamp matrix 8- custom) */
#define WPC_CUSTLAMPCOL    8  /* first custom lamp column */
#define WPC_STDSOL        28  /* Number of standard solenoids */
#define WPC_FLIPPERSOL    32  /* First flipper solenoid (8) */
#define WPC_SIMSOL        (WPC_STDSOL+1)  /* first simulated solenoid (max 3) */
#define WPC_CUSTSOL       40  /* first custom solenoid/flasher */
#define WPC_MAXPORTS       8  /* Maximum input ports */
#define WPC_CUSTINPORT     3  /* first custom inport (used in simulator) */

/*--------------------
/ Common input ports
/---------------------*/
/*-- manual switch keys --*/
#define WPC_MANSWINPORT    0
#define WPC_MANSWCOLUMNS   0x00ff
#define WPC_MANSWROWS      0xff00
/*-- common keys (start, tilt etc) --*/
#define WPC_COMINPORT      1
#define WPC_COMCOINDOORKEY 0x0100
#define WPC_COMSTARTKEY    0x0200
#define WPC_COMTILTKEY     0x0400
#define WPC_COMSTILTKEY    0x0800
#define WPC_COMLLFLIPKEY   0x1000
#define WPC_COMLRFLIPKEY   0x2000
#define WPC_COMULFLIPKEY   0x4000
#define WPC_COMURFLIPKEY   0x8000
/*-- DIPsw (and simulator) inport --*/
#define WPC_DIPINPORT      2
#define WPC_SIMINPORT      2
#define WPC_DIPSW(x)    ((x)&0xfc)

/*-------------------
/  Cabinet switches
/--------------------*/
#define swF1   111
#define swF2   112
#define swF3   113
#define swF4   114
#define swF5   115
#define swF6   116
#define swF7   117
#define swF8   118

#define swLRFlipEOS swF1
#define swLRFlip    swF2
#define swLLFlipEOS swF3
#define swLLFlip    swF4
#define swURFlipEOS swF5
#define swURFlip    swF6
#define swULFlipEOS swF7
#define swULFlip    swF8

/*-------------------
/  Common Solenoids
/--------------------*/
#define sLRFlip    (WPC_FLIPPERSOL+0)
#define sLLFlip    (WPC_FLIPPERSOL+2)
#define sURFlip    (WPC_FLIPPERSOL+4)
#define sULFlip    (WPC_FLIPPERSOL+6)

/*-- create a custom switch number --*/
/* example: #define swCustom WPC_CUSTSWNO(1,2)  // custom column 1 row 2 */
#define WPC_CUSTSWNO(c,r) ((WPC_CUSTSWCOL-1+c)*10+r)
/*-- create a custom solenoid number --*/
/* example: #define swCustom WPC_CUSTSOLNO(1)  // custom solenoid 1 */
#define WPC_CUSTSOLNO(n) (WPC_CUSTSOL-1+n)

/*-------------------------------------------
/  Draw data. draw lamps,switches,solenoids
/  in this way instead of a matrix
/--------------------------------------------*/
typedef struct {
  unsigned char x,y, color;
} wpc_tDrawData;

/*-- There are no custom fields in the game driver --*/
/*-- so I have to invent some by myself. Each driver --*/
/*-- fills in one of these in the game_init function --*/
typedef struct {
  /*-- hardware definition --*/
  int  wpcGen;        /* WPC Generation */
  char serialNo[21];  /* Securty chip serial number */
  /* number of custom switch columns, lamp columns and solenoids */
  int  swCol, lampCol, custSol;
  int  flippers;      /* flippers installed (see defines below) */
  /* inverted switches (e.g. optos) */
  unsigned char invSw[WPC_MAXSWCOL];
  /* common switches */
  struct { unsigned char start, tilt, sTilt, coinDoor, shooter; } comSw;
  int (*getSol)(int solNo);   /* get state of custom solenoid */
  /*-- Draw data --*/
  wpc_tDrawData *swData;      /* switches (NULL = standard square Matrix) */
  wpc_tDrawData *lampData;    /* lamps (NULL = standard square Matrix) */
  wpc_tDrawData *solData;     /* standard solenoids (NULL = standard square Matrix) */
  void (*drawMech)(unsigned char **line); /* draw game specific hardware */
  void *simData;              /* simulator data */
} wpc_tGameData;
extern wpc_tGameData *wpc_gameData;

/*------------------------------------------------------
/ Flipper hardware is described with the following macros
/  (macros use FLIP_LL, FLIP_LR, FLIP_UL, FLIP_UR)
/ FLIP_SW()      Flipper switches available
/ FLIP_SWNO(L,R) Flipper switch numbers if other than default (Pre-fliptronics)
/ FLIP_SOL()     CPU controlled flippers
/ Example: CPU controlled upper right flipper
/   FLIP_SW(FLIP_LL | FLIP_LR | FLIP_UR) + FLIP_SOL(FLIP_LL | FLIP_LR | FLIP_UR)
/ Example: Flippers not controlled by CPU
/   FLIP_SWNO(swFlipL, swFlipR)
/--------------------------------------------------------------*/
/*-- flipper names --*/
#define FLIP_LR        (0x1)
#define FLIP_LL        (0x2)
#define FLIP_UR        (0x4)
#define FLIP_UL        (0x8)
#define FLIP_L         (FLIP_LL | FLIP_LR)
#define FLIP_U         (FLIP_UL | FLIP_UR)

/*-- definition macros --*/
#define FLIP_BUT(x)    ((x)<<16)
#define FLIP_SW(x)     ((x)<<20)
#define FLIP_SWNO(l,r) (((l)<<8)|(r)|FLIP_SW(FLIP_L))
#define FLIP_SOL(x)    ((x)<<24)

#define FLIP_SWL(x)    (((x)>>8)&0xff)
#define FLIP_SWR(x)    ((x)&0xff)

/*---------------------------------------
/  These inports are common to all games
/----------------------------------------*/
#define WPC_PORTS \
  PORT_START /* 0 */ \
    WPCPORT_BIT(0x0001, "Column 1",  KEYCODE_Q) \
    WPCPORT_BIT(0x0002, "Column 2",  KEYCODE_W) \
    WPCPORT_BIT(0x0004, "Column 3",  KEYCODE_E) \
    WPCPORT_BIT(0x0008, "Column 4",  KEYCODE_R) \
    WPCPORT_BIT(0x0010, "Column 5",  KEYCODE_T) \
    WPCPORT_BIT(0x0020, "Column 6",  KEYCODE_Y) \
    WPCPORT_BIT(0x0040, "Column 7",  KEYCODE_U) \
    WPCPORT_BIT(0x0080, "Column 8",  KEYCODE_I) \
    WPCPORT_BIT(0x0100, "Row 1",     KEYCODE_A) \
    WPCPORT_BIT(0x0200, "Row 2",     KEYCODE_S) \
    WPCPORT_BIT(0x0400, "Row 3",     KEYCODE_D) \
    WPCPORT_BIT(0x0800, "Row 4",     KEYCODE_F) \
    WPCPORT_BIT(0x1000, "Row 5",     KEYCODE_G) \
    WPCPORT_BIT(0x2000, "Row 6",     KEYCODE_H) \
    WPCPORT_BIT(0x4000, "Row 7",     KEYCODE_J) \
    WPCPORT_BIT(0x8000, "Row 8",     KEYCODE_K) \
  PORT_START /* 1 */ \
    WPCPORT_BITDEF(  0x0001, IPT_COIN1,        IP_KEY_DEFAULT) \
    WPCPORT_BITDEF(  0x0002, IPT_COIN2,        IP_KEY_DEFAULT) \
    WPCPORT_BITDEF(  0x0004, IPT_COIN3,        KEYCODE_3) \
    WPCPORT_BITDEF(  0x0008, IPT_COIN4,        KEYCODE_4) \
    WPCPORT_BIT(     0x0010, "Escape",         KEYCODE_0) \
    WPCPORT_BIT(     0x0020, "Down",           KEYCODE_9) \
    WPCPORT_BIT(     0x0040, "Up",             KEYCODE_8) \
    WPCPORT_BIT(     0x0080, "Enter",          KEYCODE_7) \
    WPCPORT_BITTOG(  0x0100, "Coin Door",      KEYCODE_END)  \
    WPCPORT_BITDEF(  0x0200, IPT_START1,       IP_KEY_DEFAULT)  \
    WPCPORT_BITDEF(  0x0400, IPT_TILT,         KEYCODE_INSERT)  \
    WPCPORT_BIT(     0x0800, "Slam Tilt",      KEYCODE_HOME)  \
    WPCPORT_BIT(     0x1000, "Left Flipper",   KEYCODE_LCONTROL)  \
    WPCPORT_BIT(     0x2000, "Right Flipper",  KEYCODE_RCONTROL) \
    WPCPORT_BIT(     0x4000, "U Left Flipper", KEYCODE_LEFT)  \
    WPCPORT_BIT(     0x8000, "U Right Flipper",KEYCODE_RIGHT) \
  PORT_START /* 2 */ \
    WPCPORT_DIPNAME( 0x0004, 0x00, "W20") \
      WPCPORT_DIPSET(0x0000, "0" ) \
      WPCPORT_DIPSET(0x0004, "1" ) \
    WPCPORT_DIPNAME( 0x0008, 0x00, "W19") \
      WPCPORT_DIPSET(0x0000, "0" ) \
      WPCPORT_DIPSET(0x0008, "1" ) \
    WPCPORT_DIPNAME( 0x00f0, 0x00, "Country") \
      WPCPORT_DIPSET(0x0000, "USA" ) \
      WPCPORT_DIPSET(0x0010, "France 1" ) \
      WPCPORT_DIPSET(0x0020, "Germany" ) \
      WPCPORT_DIPSET(0x0030, "France 2" ) \
      WPCPORT_DIPSET(0x0080, "Export Alt" ) \
      WPCPORT_DIPSET(0x0090, "France 3" ) \
      WPCPORT_DIPSET(0x00a0, "Export" ) \
      WPCPORT_DIPSET(0x00b0, "France" ) \
      WPCPORT_DIPSET(0x00c0, "UK" ) \
      WPCPORT_DIPSET(0x00d0, "European" ) \
      WPCPORT_DIPSET(0x00e0, "Spain" ) \
      WPCPORT_DIPSET(0x00f0, "USA" )

/*---------------------
/  Exported variables
/----------------------*/
typedef struct {
  /*-- the game driver should not need to address these directly --*/
  unsigned char swMatrix[WPC_MAXSWCOL];
  unsigned char lampMatrix[WPC_MAXLAMPCOL];
} wpc_tGlobals;
extern wpc_tGlobals wpc_globals;

/*-- the internal state of the WPC chip. Should only be used in memory handlers --*/
extern unsigned char *wpc_data;

/*---------------------
/  Exported functions
/----------------------*/
/*-- switch handling, switches must be numbered as (10*column+row) --*/
extern void wpc_setSw(int swNo, int value);
extern int wpc_getSw(int swNo);
extern void wpc_updInvSw(int swNo, int inv);

/*-- solenoid handling --*/
extern int wpc_getSol(int solNo);

/*-- use this if a fallback is required in a custom memory handler --*/
extern READ_HANDLER(wpc_r);
extern WRITE_HANDLER(wpc_w);

/*-- output text on the display */
void wpc_textOut(char *buf, int length, int x, int y, int color);
void CLIB_DECL wpc_textOutf(int x, int y, int color, char *text, ...);

/*-- These are only here so the game structure can be in the game file --*/
extern struct MachineDriver machine_driver_wpcDMD;
extern struct MachineDriver machine_driver_wpcAlpha;
#define wpc_mAlpha      wpcAlpha
#define wpc_mFliptron   wpcDMD
#define wpc_mDMD        wpcDMD
#define wpc_mDCS        wpcDMD
#define wpc_mSecurity   wpcDMD
#define wpc_m95         wpcDMD

#ifdef WPC_WPCSOUND
   extern struct MachineDriver machine_driver_wpcDMD_s;
   extern struct MachineDriver machine_driver_wpcAlpha_s;
   #define wpc_mAlphaS      wpcAlpha_s
   #define wpc_mFliptronS   wpcDMD_s 
   #define wpc_mDMDS        wpcDMD_s
#else /* WPC_WPCSOUND */
   #define wpc_mAlphaS      wpcAlpha
   #define wpc_mFliptronS   wpcDMD
   #define wpc_mDMDS        wpcDMD
#endif /* WPC_WPCSOUND */

#ifdef WPC_DCSSOUND
   extern struct MachineDriver machine_driver_wpcDCS_s;
   extern struct MachineDriver machine_driver_wpcDCS95_s;
   #define wpc_mDCSS        wpcDCS_s
   #define wpc_mSecurityS   wpcDCS_s
   #define wpc_m95S         wpcDCS95_s
#else /* WPC_DCSSOUND */
   #define wpc_mDCSS        wpcDMD
   #define wpc_mSecurityS   wpcDMD
   #define wpc_m95S         wpcDMD
#endif /* WPC_DCSSOUND */

/*-- define the game --*/
#define WPC_GAMEDEF(name, ver, longname, year, manuf, machine, flag) \
  GAMEX(year,name##_##ver,0,machine,name,name,ROT0,manuf,longname,flag)
#define WPC_CLONEDEF(name, ver, clonever, longname, year, manuf, machine,flag) \
  GAMEX(year,name##_##ver,name##_##clonever,machine,name,name,ROT0,manuf,longname,flag)

/*-- Check if Sound is being used --*/
#define USESOUND (FORCESOUNDCPU || (!(Machine->gamedrv->flags & GAME_NO_SOUND) && Machine->sample_rate))



#endif /* INC_WPC */

