#include "driver.h"
#include "cpu/adsp2100/adsp2100.h"
#include "wpc.h"
#include "dcs.h"

#define DISABLE_DCS_SPEEDUP	0
/* Currently it works only for wc,dm,fs,jm,tom pinballs */


/*-- ADSP core functions --*/
static void adsp_boot(void);
static void adsp_txCallback(int port, INT32 data);
static WRITE16_HANDLER(adsp_control_w);

/*-- bank handlers --*/
static WRITE16_HANDLER(dcs_ROMbankSelect1_w);
static WRITE16_HANDLER(dcs_ROMbankSelect2_w);
static WRITE16_HANDLER(dcs_RAMbankSelect_w);
static READ16_HANDLER(dcs_RAMbankSelect_r);
static READ16_HANDLER(dcs1_ROMbank_r);
static READ16_HANDLER(dcs2_ROMbank_r);
static READ16_HANDLER(dcs2_RAMbank_r);
static WRITE16_HANDLER(dcs2_RAMbank_w);

/*-- sound interface handlers --*/
/*static*/ READ16_HANDLER(dcs_latch_r);
/*static*/ WRITE16_HANDLER(dcs_latch_w);

/*-- sound generation --*/
static int dcs_custStart(const struct MachineSound *msound);
static void dcs_custStop(void);
static void dcs_dacUpdate(int num, INT16 *buffer, int length);

/*-- local data --*/
#define DCS_BUFFER_SIZE	  4096
#define DCS_BUFFER_MASK	  (DCS_BUFFER_SIZE - 1)

static struct {
  int    enabled;
  UINT32 sOut, sIn, sStep;
  INT16 *buffer;
  int    stream;
} dcs_dac;

static struct {
 UINT16  ROMbank1;
 UINT16  ROMbank2;
 UINT16  RAMbank;
 int     replyAvail;
} locals;

static struct {
  UINT16 start;
  UINT16 size, step;
  int    sRate;
  int    iReg;
} adsp_aBufData;

/*--------------
/  Memory maps
/---------------*/
MEMORY_READ16_START(dcs2_readmem)
  { ADSP_DATA_ADDR_RANGE(0x0000, 0x07ff), dcs2_ROMbank_r },
  { ADSP_DATA_ADDR_RANGE(0x1000, 0x1fff), MRA16_RAM },
  { ADSP_DATA_ADDR_RANGE(0x2000, 0x2fff), dcs2_RAMbank_r },
  { ADSP_DATA_ADDR_RANGE(0x3200, 0x3200), dcs_RAMbankSelect_r },
  { ADSP_DATA_ADDR_RANGE(0x3300, 0x3300), dcs_latch_r },
  { ADSP_DATA_ADDR_RANGE(0x3800, 0x39ff), MRA16_RAM },
  { ADSP_PGM_ADDR_RANGE (0x0000, 0x0800), MRA16_RAM }, /* Internal boot RAM */
  { ADSP_PGM_ADDR_RANGE (0x1000, 0x3fff), MRA16_RAM }, /* External RAM */
MEMORY_END

MEMORY_WRITE16_START(dcs2_writemem)
  { ADSP_DATA_ADDR_RANGE(0x1000, 0x1fff), MWA16_RAM },
  { ADSP_DATA_ADDR_RANGE(0x2000, 0x2fff), dcs2_RAMbank_w },
  { ADSP_DATA_ADDR_RANGE(0x3000, 0x3000), dcs_ROMbankSelect1_w },
  { ADSP_DATA_ADDR_RANGE(0x3100, 0x3100), dcs_ROMbankSelect2_w },
  { ADSP_DATA_ADDR_RANGE(0x3200, 0x3200), dcs_RAMbankSelect_w },
  { ADSP_DATA_ADDR_RANGE(0x3300, 0x3300), dcs_latch_w },
  { ADSP_DATA_ADDR_RANGE(0x3800, 0x39ff), MWA16_RAM },
  { ADSP_DATA_ADDR_RANGE(0x3fe0, 0x3fff), adsp_control_w },
  { ADSP_PGM_ADDR_RANGE (0x0000, 0x0800), MWA16_RAM }, /* Internal boot RAM */
  { ADSP_PGM_ADDR_RANGE (0x1000, 0x3fff), MWA16_RAM }, /* External RAM */
MEMORY_END

MEMORY_READ16_START(dcs1_readmem)
  { ADSP_DATA_ADDR_RANGE(0x0000, 0x1fff), MRA16_RAM },
  { ADSP_DATA_ADDR_RANGE(0x2000, 0x2fff), dcs1_ROMbank_r },
  { ADSP_DATA_ADDR_RANGE(0x3800, 0x39ff), MRA16_RAM },
  { ADSP_PGM_ADDR_RANGE (0x0000, 0x0800), MRA16_RAM }, /* Internal boot RAM */
  { ADSP_PGM_ADDR_RANGE (0x1000, 0x2fff), MRA16_RAM }, /* External RAM */
  { ADSP_PGM_ADDR_RANGE (0x3000, 0x3000), dcs_latch_r }, /* doesn't work */
MEMORY_END

MEMORY_WRITE16_START(dcs1_writemem)
  { ADSP_DATA_ADDR_RANGE(0x0000, 0x1fff), MWA16_RAM },
  { ADSP_DATA_ADDR_RANGE(0x3000, 0x3000), dcs_ROMbankSelect1_w },
  { ADSP_DATA_ADDR_RANGE(0x3800, 0x39ff), MWA16_RAM },
  { ADSP_DATA_ADDR_RANGE(0x3fe0, 0x3fff), adsp_control_w },
  { ADSP_PGM_ADDR_RANGE (0x0000, 0x0800), MWA16_RAM },
  { ADSP_PGM_ADDR_RANGE (0x1000, 0x2fff), MWA16_RAM },
  { ADSP_PGM_ADDR_RANGE (0x3000, 0x3000), dcs_latch_w }, /* doesn't work */
MEMORY_END

/*---------------
/  Bank handlers
/----------------*/
#define DCS1_ROMBANKBASE(bank) \
  ((UINT8 *)(memory_region(WPC_MEMREG_SROM) + (((bank) & 0x7ff)<<12)))
#define DCS2_ROMBANKBASE(bankH, bankL) \
  ((UINT8 *)(memory_region(WPC_MEMREG_SROM) + \
             (((bankH) & 0x1c)<<18) + (((bankH) & 0x01)<<19) + (((bankL) & 0xff)<<11)))
#define DCS2_RAMBANKBASE(bank) \
  ((UINT16 *)(((bank) & 0x08) ? memory_region(WPC_MEMREG_SBANK) : \
              (memory_region(WPC_MEMREG_SCPU) + ADSP2100_DATA_OFFSET + (0x2000<<1))))

static WRITE16_HANDLER(dcs_ROMbankSelect1_w)
{
	locals.ROMbank1 = data;

	/* they write 0x800 here just before entering the stall loop */
#if (!DISABLE_DCS_SPEEDUP)
      if (wpc_gameData->wpcGen < WPC_GEN95)
	if (data == 0x800)
	{
		/* calculate the next buffer address */
		int source = cpu_get_reg( ADSP2100_I0 + adsp_aBufData.iReg );
		int ar = source + adsp_aBufData.size / 2;

		/* check for wrapping */
		if ( ar >= ( adsp_aBufData.start + adsp_aBufData.size ) )
			ar = adsp_aBufData.start;

		/* set it */
		cpu_set_reg( ADSP2100_AR, ar );

		/* go around the buffer syncing code, we sync manually */

		cpu_set_reg( ADSP2100_PC, cpu_get_pc() + 8 );
		cpu_spinuntil_int();
	}
#endif

}
static WRITE16_HANDLER(dcs_ROMbankSelect2_w) { locals.ROMbank2 = data; }
static WRITE16_HANDLER(dcs_RAMbankSelect_w)  { locals.RAMbank  = data; }
static READ16_HANDLER (dcs_RAMbankSelect_r)  { return locals.RAMbank; }

static READ16_HANDLER(dcs1_ROMbank_r)
  { return DCS1_ROMBANKBASE(locals.ROMbank1)[offset]; }

static READ16_HANDLER(dcs2_ROMbank_r)
  { return DCS2_ROMBANKBASE(locals.ROMbank2,locals.ROMbank1)[offset]; }

static READ16_HANDLER(dcs2_RAMbank_r)
  { return DCS2_RAMBANKBASE(locals.RAMbank)[offset]; }

static WRITE16_HANDLER(dcs2_RAMbank_w)
  { DCS2_RAMBANKBASE(locals.RAMbank)[offset] = data; }

static UINT32 *dcs_getBootROM(void) {
  return (UINT32 *)(memory_region(WPC_MEMREG_SROM) + ((locals.ROMbank1 & 0xff)<<12));
}

/*----------------------
/ Other memory handlers
/-----------------------*/
/* These should be static but the patched ADSP core requires them */

/*static*/ READ16_HANDLER(dcs_latch_r) {
  cpu_set_irq_line(WPC_SCPUNO, ADSP2105_IRQ2, CLEAR_LINE);
  return soundlatch_r(0);
}

/*static*/ WRITE16_HANDLER(dcs_latch_w) {
  DBGLOG(("dcs_latch_w: %4x\n",data));
  soundlatch2_w(0, data);
  locals.replyAvail = TRUE;
}

/*----------------
/ Sound interface
/-----------------*/
struct CustomSound_interface dcs_custInt = {
  dcs_custStart, dcs_custStop, 0
};

static int dcs_custStart(const struct MachineSound *msound){
  /* allocate a DAC stream */
  dcs_dac.stream = stream_init("DCS DAC", 100, Machine->sample_rate, 0, dcs_dacUpdate);

  /* allocate memory for our buffer */
  dcs_dac.buffer = malloc(DCS_BUFFER_SIZE * sizeof(INT16));

  return (dcs_dac.buffer == 0);
}

static void dcs_custStop(void) {
  if (dcs_dac.buffer)
    { free(dcs_dac.buffer); dcs_dac.buffer = NULL; }
}

static void dcs_dacUpdate(int num, INT16 *buffer, int length)
{
	UINT32 current, step, indx;
	INT16 *source;
	int i;

	/* DAC generation */
	if (dcs_dac.enabled)
	{
		source = dcs_dac.buffer;
		current = dcs_dac.sOut;
		step = dcs_dac.sStep;

		/* fill in with samples until we hit the end or run out */
		for (i = 0; i < length; i++)
		{
			indx = current >> 16;
			if (indx >= dcs_dac.sIn)
				break;
			current += step;
			*buffer++ = source[indx & DCS_BUFFER_MASK];
		}

		/* fill the rest with the last sample */
		for ( ; i < length; i++)
			*buffer++ = source[(dcs_dac.sIn - 1) & DCS_BUFFER_MASK];

		/* mask off extra bits */
		while (current >= (DCS_BUFFER_SIZE << 16))
		{
			current -= DCS_BUFFER_SIZE << 16;
			dcs_dac.sIn -= DCS_BUFFER_SIZE;
		}

		/* update the final values */
		dcs_dac.sOut = current;
	}
	else
		memset(buffer, 0, length * sizeof(INT16));
}

/*-------------------
/ Exported interface
/---------------------*/
READ_HANDLER(dcs_data_r) {
  locals.replyAvail = FALSE;
  return soundlatch2_r(0) & 0xff;
}

WRITE_HANDLER(dcs_data_w) {
  #ifdef MAME_DEBUG
  char str[50];
  sprintf(str,"%4d",data);
  wpc_textOut(str,strlen(str),10,100,BLACK);
  #endif

  logerror("dcs_data_w: %4d\n",data);
  soundlatch_w(0, data);
  cpu_set_irq_line(WPC_SCPUNO, ADSP2105_IRQ2, ASSERT_LINE);
}

READ_HANDLER(dcs_ctrl_r) {
  return locals.replyAvail ? 0x80 : 0x00;
}

/*-- handle bug in ADSP core */
static OPBASE_HANDLER(opbaseoveride) { return -1; }

void dcs_init(void) {

  cpu_setOPbaseoverride(WPC_SCPUNO, opbaseoveride);
  /* initialize our state structure */
  memset(&locals, 0, sizeof(locals));

  /* reset DAC generation */
  dcs_dac.sStep = 0x10000;
  dcs_dac.sIn = 0;
  dcs_dac.sOut = 0;

  /* initialize the ADSP Tx callback */
  adsp2105_set_tx_callback(adsp_txCallback);

  /* clear all interrupts */
  cpu_set_irq_line(WPC_SCPUNO, ADSP2105_IRQ0, CLEAR_LINE );
  cpu_set_irq_line(WPC_SCPUNO, ADSP2105_IRQ1, CLEAR_LINE );
  cpu_set_irq_line(WPC_SCPUNO, ADSP2105_IRQ2, CLEAR_LINE );

  /* boot */
  adsp_boot();
}

/*--------------------------------------------------*/
/*-- This should actually be part of the CPU core --*/
/*--------------------------------------------------*/
enum {
  S1_AUTOBUF_REG = 15, S1_RFSDIV_REG, S1_SCLKDIV_REG, S1_CONTROL_REG,
  S0_AUTOBUF_REG, S0_RFSDIV_REG, S0_SCLKDIV_REG, S0_CONTROL_REG,
  S0_MCTXLO_REG, S0_MCTXHI_REG, S0_MCRXLO_REG, S0_MCRXHI_REG,
  TIMER_SCALE_REG, TIMER_COUNT_REG, TIMER_PERIOD_REG, WAITSTATES_REG,
  SYSCONTROL_REG
};

static struct {
 UINT16  ctrlRegs[32];
 void   *irqTimer;
 UINT32 *(*getBootROM)(void);
 void   (*txData)(UINT16 start, UINT16 size, UINT16 step, int sRate);
} adsp = {{0}, NULL, dcs_getBootROM, NULL};

static void adsp_boot(void) {
  UINT32 *src = adsp.getBootROM();
  UINT32 *dst = (UINT32 *)(memory_region(WPC_MEMREG_SCPU) + ADSP2100_PGM_OFFSET);
  UINT32  data = src[0];
  UINT32  size;
  UINT32  ii;

  data = src[0];
#ifdef LSB_FIRST // ************** not really tested yet ****************
  data = ((data & 0xff)<<24) | ((data & 0xff00)<<8) | ((data>>8) & 0xff00) | ((data>>24) & 0xff);
#endif
  size = ((data & 0xff) + 1) * 8;

  for (ii = 0; ii < size; ii++) {
    data = src[ii];
#ifdef LSB_FIRST // ************** not really tested yet ****************
    data = ((data & 0xff)<<24) | ((data & 0xff00)<<8) | ((data>>8) & 0xff00) | ((data>>24) & 0xff);
#endif
    data >>= 8;
    dst[ii] = data;
  }
  if (adsp.irqTimer)
    { timer_remove(adsp.irqTimer); adsp.irqTimer = NULL; }
}

static WRITE16_HANDLER(adsp_control_w) {

  adsp.ctrlRegs[offset] = data;
  switch (offset) {
    case SYSCONTROL_REG:
      if (data & 0x0200) {
        /* boot force */
        logerror("boot force\n");
        cpu_set_reset_line(WPC_SCPUNO, PULSE_LINE);
        adsp_boot();
        adsp.ctrlRegs[SYSCONTROL_REG] &= ~0x0200;
      }
      /* see if SPORT1 got disabled */
      if ((data & 0x0800) == 0) {
	 stream_update(dcs_dac.stream, 0);
        /* nuke the timer */
        if (adsp.irqTimer)
          { timer_remove(adsp.irqTimer); adsp.irqTimer = NULL; }
      }
      break;
    case S1_AUTOBUF_REG:
      /* autobuffer off: nuke the timer */
      stream_update(dcs_dac.stream, 0);
      if ((data & 0x0002) == 0) {

        /* nuke the timer */
        if (adsp.irqTimer)
          { timer_remove(adsp.irqTimer); adsp.irqTimer = NULL; }
      }
      break;
    case S1_CONTROL_REG:
      if (((data>>4) & 3) == 2)
	logerror("Oh no!, the data is compresed with u-law encoding\n");
      if (((data>>4) & 3) == 3)
	logerror("Oh no!, the data is compresed with A-law encoding\n");
      break;
  } /* switch */
}

static void adsp_irqGen(int state)
{
	/* get the index register */
	int reg = cpunum_get_reg(WPC_SCPUNO, ADSP2100_I0 + adsp_aBufData.iReg );

	/* translate into data memory bus address */
	int source = ADSP2100_DATA_OFFSET + (reg << 1);
	UINT8 *mem = (UINT8 *)(memory_region(WPC_MEMREG_SCPU));
	int i;

	/* copy the current data into the buffer */
	for (i = 0; i < (adsp_aBufData.size / adsp_aBufData.step / 2); i++)
		dcs_dac.buffer[dcs_dac.sIn++ & DCS_BUFFER_MASK] = READ_WORD(&mem[source + (adsp_aBufData.step * i * 2) ]);

	/* increment it */
	reg += adsp_aBufData.step * (adsp_aBufData.size / adsp_aBufData.step / 2);

	/* check for wrapping */
	if ( reg >= ( adsp_aBufData.start + adsp_aBufData.size ) )
	{
		/* reset the base pointer */
		reg = adsp_aBufData.start;

		/* generate the (internal, thats why the pulse) irq */
		cpu_set_irq_line( WPC_SCPUNO, ADSP2105_IRQ1, PULSE_LINE );
	}

	/* store it */
	cpunum_set_reg( WPC_SCPUNO, ADSP2100_I0 + adsp_aBufData.iReg, reg );

#if (!DISABLE_DCS_SPEEDUP)
      if (wpc_gameData->wpcGen < WPC_GEN95)
	/* this is the same trigger as an interrupt */
	cpu_trigger( -2000 + WPC_SCPUNO );
#endif
}

static void adsp_txCallback(int port, INT32 data)
{
  if (port != 1)
    { logerror("tx0 not handled\n"); return; };

  if ((adsp.ctrlRegs[SYSCONTROL_REG] & 0x0800) == 0)
    logerror("tx1 without SPORT1 enabled\n");
  else if ((adsp.ctrlRegs[S1_AUTOBUF_REG] & 0x0002) == 0)
    logerror("SPORT1 without autobuffer");
  else {
    int ireg, mreg;

    stream_update(dcs_dac.stream, 0);

    ireg = (adsp.ctrlRegs[S1_AUTOBUF_REG]>>9) & 7;
    mreg = ((adsp.ctrlRegs[S1_AUTOBUF_REG]>>7) & 3) | (ireg & 0x04);


    /* start = In, size = Ln, step = Mn */
    adsp_aBufData.step  = cpu_get_reg(ADSP2100_M0 + mreg);
    adsp_aBufData.size  = cpu_get_reg(ADSP2100_L0 + ireg);

    /*-- assume that the first sample comes from the memory position before --*/
    adsp_aBufData.start = cpu_get_reg(ADSP2100_I0 + ireg) - adsp_aBufData.step;

    cpunum_set_reg(WPC_SCPUNO, ADSP2100_I0 + adsp_aBufData.iReg, adsp_aBufData.start);
    adsp_aBufData.sRate = (Machine->drv->cpu[WPC_SCPUNO].cpu_clock /
                          (2 * (adsp.ctrlRegs[S1_SCLKDIV_REG] + 1))) / 16;
    adsp_aBufData.iReg = ireg;

    dcs_dac.enabled = 1;
    dcs_dac.sIn = 0;
    dcs_dac.sOut = 0;
    dcs_dac.sStep = (int)(adsp_aBufData.sRate * 65536.0/ (double)Machine->sample_rate);

    /*-- fire the irq timer --*/
    adsp.irqTimer = timer_pulse(TIME_IN_HZ(adsp_aBufData.sRate) *
                                (adsp_aBufData.size / adsp_aBufData.step / 2), 0,
                                adsp_irqGen);
    return;
  }

  /*-- if we get there, something went wrong. Disable transmission --*/
  stream_update(dcs_dac.stream, 0);

  dcs_dac.enabled = 0;

  /*-- remove any pending timer --*/
  if (adsp.irqTimer)
    { timer_remove(adsp.irqTimer); adsp.irqTimer = NULL; }
}

