#include "driver.h"
#include "wpc.h"
#include "wpcsim.h"
#include "dcs.h"
#include "wpcsound.h"

/* 061100 Added Popeye, BOP, and Safe Cracker Sound Support (SJE) */
/* 051100 Added TAF Sound Support (SJE) */
/* 021100 Added Party Zone Sound Support (SJE) */
/* 011100 Moved Funhouse and T2 To Own Simulator Files */
/* 301000 Added WCS94 Sound Support (SJE) */
/* 291000 Added T2 and WW Sound Support (SJE) */
/* 171000 Added FT sound, NBA S3.0 */
/* 181000 Corrected games year and names */
#define swStart     13
#define swTilt      14
#define swSlamTilt  21
#define swCoinDoor  22

#define INITGAME(name, balls) \
static void init_##name(void) { \
  wpc_gameData = &name##GameData; \
} \
WPC_INPUT_PORTS_START(name, balls) WPC_INPUT_PORTS_END

/*---------------
/  Cactus Canyon
/----------------*/
static wpc_tGameData ccGameData = {
  WPC_GEN95,
  "566 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x7f, 0x03, 0x00, 0x00, 0xc1, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(cc,3/*?*/)

WPC_ROMSTART(cc,12,"cc_g11.1_2",0x100000,0x17ad9266)
DCS_SOUNDROM6m("sav2_8.rom",0x94928841,
               "sav3_8.rom",0xa22b13f0,
               "sav4_8.rom",0xfe8324e2,
               "sav5_8.rom",0x1b2a1ff3,
	       "sav6_8.rom",0x2cccf10e,
	       "sav7_8.rom",0x90fb1277)
WPC_ROMEND

WPC_ROMSTART(cc,13,"cc_g11.1_3",0x100000,0x7741fa4e)
DCS_SOUNDROM6m("sav2_8.rom",0x94928841,
               "sav3_8.rom",0xa22b13f0,
	       "sav4_8.rom",0xfe8324e2,
	       "sav5_8.rom",0x1b2a1ff3,
	       "sav6_8.rom",0x2cccf10e,
	       "sav7_8.rom",0x90fb1277)
WPC_ROMEND

WPC_GAMEDEF (cc,13,   "CC - Cactus Canyon 1.3 (S1.0)",1998,"Bally",wpc_m95S,0)
WPC_CLONEDEF(cc,12,13,"CC - Cactus Canyon 1.2 (S1.0)",1998,"Bally",wpc_m95S,0)


/*===========
/  Dr. Dude
/============*/
static wpc_tGameData ddGameData = {
  WPC_GENALPHA,
  {0}, /* no serial number */
  0,0,0,
  FLIP_SWNO(12,11),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(dd,3/*?*/)

WPC_ROMSTART(dd,p7,"dude_u6.p7",0x020000,0xb6c35b98) WPC_ROMEND
// uses system 11 sound

WPC_GAMEDEF(dd,p7,"DD - Dr. Dude P-7",1990,"Bally",wpc_mAlpha,GAME_NO_SOUND)


/*--------------
/ Indiana Jones
/---------------*/
static wpc_tGameData ijGameData = {
  WPC_GENDCS,
  {0}, /* sNO */
  0,0,0,
  FLIP_SW(FLIP_L) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x5f, 0x00, 0x00, 0x06, 0x7f, 0x00, 0x00, 0x70 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(ij,3/*?*/)

WPC_ROMSTART(ij,l7,"ijone_l7.rom",0x80000,0x4658c877) WPC_ROMEND

WPC_GAMEDEF(ij,l7,"IJ - Indiana Jones L-7",1993,"Williams",wpc_mDCS,GAME_NO_SOUND)

/*--------------
/ Monster Bash
/---------------*/
static wpc_tGameData mbGameData = {
  WPC_GEN95,
  "565 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x3f, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(mb,3/*?*/)

WPC_ROMSTART(mb,106,"mb_1_06.bin",0x100000,0x381a8822) WPC_ROMEND
WPC_ROMSTART(mb,10, "mb_g11.1_0", 0x100000,0x6b8db967) WPC_ROMEND

WPC_GAMEDEF (mb,106,   "MB - Monster Bash 1.06",1998,"Williams",wpc_m95,GAME_NO_SOUND)
WPC_CLONEDEF(mb,10,106,"MB - Monster Bash 1.0", 1998,"Williams",wpc_m95,GAME_NO_SOUND)

/*------------------------------
/ Star Trek the Next Generation
/-------------------------------*/
static wpc_tGameData sttngGameData = {
  WPC_GENDCS,
  "",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(sttng,3/*?*/)

WPC_ROMSTART(sttng,l7,"trek_lx7.rom",0x80000,0xd439fdbb)
DCS_SOUNDROM7x("ng_u2_s.l1",0xc3bd7bf5,
               "ng_u3_s.l1",0x9456cac7,
               "ng_u4_s.l1",0x179d22a4,
               "ng_u5_s.l1",0x231a3e72,
               "ng_u6_s.l1",0xbb21377d,
               "ng_u7_s.l1",0xd81b39f0,
               "ng_u8_s.l1",0xc9fb065e)
WPC_ROMEND
WPC_GAMEDEF(sttng,l7,"STTNG - Star Trek: The Next Generation Lx-7",1993,"Williams",wpc_mDCSS,0)

/*--------------
/ The Shadow
/---------------*/
static wpc_tGameData tsGameData = {
  WPC_GENSECURITY,
  "532 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_UR) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0xe7, 0x7f, 0x00, 0x00, 0x00, 0xf0, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(ts,3/*?*/)

WPC_ROMSTART(ts,lx6,"shad_x6.rom",0x080000,0x0a72268d) WPC_ROMEND
WPC_ROMSTART(ts,lx5,"shad_x5.rom",0x080000,0xbb545f83) WPC_ROMEND

WPC_GAMEDEF (ts,lx6,    "TS - The Shadow Lx-6",1994,"Bally",wpc_mSecurity,GAME_NO_SOUND)
WPC_CLONEDEF(ts,lx5,lx6,"TS - The Shadow Lx-5",1994,"Bally",wpc_mSecurity,GAME_NO_SOUND)

/*--------------
/ Theatre of Magic
/---------------*/
static wpc_tGameData tomGameData = {
  WPC_GENSECURITY,
  "539 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x3f, 0x00, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(tom,4/*?*/)

WPC_ROMSTART(tom,13,"tom1_3x.rom",0x80000,0xaff4d14c)
DCS_SOUNDROM6x("tm_u2_s.l2",0xb128fbba,
               "tm_u3_s.l2",0x128c7d3c,
               "tm_u4_s.l2",0x3d9b2354,
               "tm_u5_s.l2",0x44247b60,
               "tm_u6_s.l2",0xf366bbe5,
               "tm_u7_s.l2",0xf98e9e38)
WPC_ROMEND
WPC_GAMEDEF(tom,13,"TOM - Theatre of Magic 1.3",1995,"Bally",wpc_mSecurityS,0)


/*--------------
/ World Cup Soccer
/---------------*/
static wpc_tGameData wcsGameData = {
  WPC_GENSECURITY,
  "531 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x3f, 0x1f, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(wcs,3/*?*/)

WPC_ROMSTART(wcs,l2,"wcup_lx2.rom",0x80000,0x0e4514e8)
DCS_SOUNDROM7x("wcup_u2.rom",0x92252f28,
               "wcup_u3.rom",0x83f541ad,
               "wcup_u4.rom",0x1540c505,
               "wcup_u5.rom",0xbddad8d4,
               "wcup_u6.rom",0x00f46c12,
               "wcup_u7.rom",0xfff01703,
               "wcup_u8.rom",0x670cd382)
WPC_ROMEND

WPC_GAMEDEF(wcs,l2,"WCS - World Cup Soccer Lx-2",1994,"Bally",wpc_mSecurityS,0)

/*----------------
/ Bride Of Pinbot
/-----------------*/
static wpc_tGameData bopGameData = {
  WPC_GENALPHA,
  "",
  0,0,0,
  FLIP_SWNO(11,12),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(bop,3/*?*/)

WPC_ROMSTART(bop,l7,"tmbopl_7.rom",0x40000,0x773e1488) 
WPCS_SOUNDROM222("mach_u18.l1",0xf3f53896,
                 "mach_u15.l1", 0xfb49513b,
                 "mach_u14.l1", 0xbe2a736a)
WPC_ROMEND

WPC_GAMEDEF(bop,l7,"BOP - The Machine: Bride of Pinbot L-7",1991,"Williams",wpc_mAlphaS,0)

/*----------------
/ Medeival Madness
/-----------------*/
static wpc_tGameData mmGameData = {
  WPC_GEN95,
  "559 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x7f, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(mm,3/*?*/)

WPC_ROMSTART(mm,109,"mm_1_09.bin",0x100000,0x9bac4d0c)
DCS_SOUNDROM5xm("mm_s2.1_0",  0xc55c3b71,
                "mm_sav3.rom",0xed1be570,
                "mm_sav4.rom",0x9c89eacf,
                "mm_sav5.rom",0x45089e30,
                "mm_sav6.rom",0x439d55f2)
WPC_ROMEND
WPC_ROMSTART(mm,10, "mm_g11.1_0", 0x080000,0x6bd735c6)
DCS_SOUNDROM5xm("mm_s2.1_0",  0xc55c3b71,
                "mm_sav3.rom",0xed1be570,
                "mm_sav4.rom",0x9c89eacf,
                "mm_sav5.rom",0x45089e30,
                "mm_sav6.rom",0x439d55f2)
WPC_ROMEND

WPC_GAMEDEF (mm,109,   "MM - Medieval Madness 1.09",1997,"Williams",wpc_m95S,0)
WPC_CLONEDEF(mm,10,109,"MM - Medieval Madness 1.0", 1997,"Williams",wpc_m95S,0)

/*----------------
/ Black Rose
/-----------------*/
static wpc_tGameData brGameData = {
  WPC_GENFLIPTRON,
  "",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_UR) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(br,3/*?*/)

WPC_ROMSTART(br,l4,"brose_l4.rom",0x80000,0xe140dc46) WPC_ROMEND

WPC_GAMEDEF (br,l4,"BR - Black Rose L-4",1992,"Bally",wpc_mFliptron,GAME_NO_SOUND)

/*----------------
/ Creature from black lagoon
/-----------------*/
static wpc_tGameData cftblGameData = {
  WPC_GENFLIPTRON,
  "",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x50, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(cftbl,3/*?*/)

WPC_ROMSTART(cftbl,l4,"creat_l4.rom",0x80000,0xb8778cb6)
WPCS_SOUNDROM288("bl_u18.l1",0x87267bcc,
                 "bl_u15.l1",0x15477d6f,
                 "bl_u14.l1",0x6b02bee4)
WPC_ROMEND

WPC_GAMEDEF (cftbl,l4,"CFTBL - Creature from the Black Lagoon L-4",1992,"Bally",wpc_mFliptronS,0)
/*----------------
/ Congo
/-----------------*/
static wpc_tGameData congoGameData = {
  WPC_GEN95,
  "550 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_UL),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x3f, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(congo,3/*?*/)

WPC_ROMSTART(congo,21,"cg_g11.2_1",0x80000,0x5d8435bf) WPC_ROMEND

WPC_GAMEDEF (congo,21,"Congo 2.1",1995,"Williams",wpc_m95,GAME_NO_SOUND)

/*--------------
/ Corvette
/---------------*/
static wpc_tGameData corvGameData = {
  WPC_GENSECURITY,
  "536 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_UL),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x7f, 0x07, 0x33, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(corv,3/*?*/)

WPC_ROMSTART(corv,21,"corv_2_1.rom",0x80000,0x4fe64c6d) WPC_ROMEND

WPC_GAMEDEF(corv,21,"Corvette 2.1",1994,"Bally",wpc_mSecurity,GAME_NO_SOUND)

/*----------------
/ The Champion Pub
/-----------------*/
static wpc_tGameData cpGameData = {
  WPC_GEN95,
  "563 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x3f, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(cp,3/*?*/)

WPC_ROMSTART(cp,16,"cp_g11.1_6",0x80000,0xd6d0b921) WPC_ROMEND

WPC_GAMEDEF(cp,16,"CP - The Champion Pub 1.6",1998,"Bally",wpc_m95,GAME_NO_SOUND)

/*----------------
/ Cirqus Voltaire
/-----------------*/
static wpc_tGameData cvGameData = {
  WPC_GEN95,
  "562 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(cv,3/*?*/)

WPC_ROMSTART(cv,14,"cirq_14.rom",0x100000,0x7a8bf999) WPC_ROMEND

WPC_GAMEDEF(cv,14,"CV - Cirqus Voltaire 1.4",1997,"Bally",wpc_m95,GAME_NO_SOUND)

/*--------------
/ Dirty Harry
/---------------*/
static wpc_tGameData dhGameData = {
  WPC_GENSECURITY,
  "530 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0xff, 0x7d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(dh,3/*?*/)

WPC_ROMSTART(dh,lx2,"harr_lx2.rom",0x80000,0xd92c2d35) WPC_ROMEND

WPC_GAMEDEF(dh,lx2,"DH - Dirty Harry Lx-2",1995,"Williams",wpc_mSecurity,GAME_NO_SOUND)

/*--------------
/ Demolition Man
/---------------*/
static wpc_tGameData dmGameData = {
  WPC_GENDCS,
  {0}, /* sNO */
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_UL),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x30, 0x3f, 0x00, 0x00, 0x40, 0x2f, 0x00, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(dm,3/*?*/)

WPC_ROMSTART(dm,lx4,"dman_lx4.rom",0x80000,0xc2d0f493)
DCS_SOUNDROM6x("dm_u2_s.l2",0x85fb8bce,
               "dm_u3_s.l2",0x2b65a66e,
               "dm_u4_s.l2",0x9d6815fe,
               "dm_u5_s.l2",0x9f614c27,
               "dm_u6_s.l2",0x3efc2c0e,
               "dm_u7_s.l2",0x75066af1)
WPC_ROMEND

WPC_GAMEDEF(dm,lx4,"DM - Demolition Man Lx-4",1994,"Williams",wpc_mDCSS,0)

/*--------------
/ Dracula
/---------------*/
static wpc_tGameData dracGameData = {
  WPC_GENFLIPTRON,
  {0}, /* sNO */
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x7f, 0x00, 0x07, 0x02, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(drac,3/*?*/)

WPC_ROMSTART(drac,l1,"dracu_l1.rom",0x80000,0xb8679686) WPC_ROMEND

WPC_GAMEDEF(drac,l1,"Bram Stoker's Dracula L-1",1993,"Williams",wpc_mFliptron,GAME_NO_SOUND)

/*--------------
/ Doctor Who
/---------------*/
static wpc_tGameData dwGameData = {
  WPC_GENFLIPTRON,
  {0}, /* sNO */
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_UL),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x05, 0x00, 0x00, 0x00, 0x7f, 0x01, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(dw,3/*?*/)

WPC_ROMSTART(dw,l2,"drwho_l2.rom",0x80000,0x988c2deb)
WPCS_SOUNDROM288("dw_u18.l1",0x1ca78024,
                 "dw_u15.l1",0x9b87ff26,
                 "dw_u14.l1",0x71f7d55b)
WPC_ROMEND

WPC_GAMEDEF(dw,l2,"DW - Doctor Who L-2",1992, "Bally",wpc_mFliptronS,0)

/*--------------
/ Flintstones
/---------------*/
static wpc_tGameData fsGameData = {
  WPC_GENSECURITY,
  "529 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_UR) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x3f, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(fs,3/*?*/)

WPC_ROMSTART(fs,lx5,"flin_lx5.rom",0x80000,0x06707244)
DCS_SOUNDROM8x("fs_u2_s.l1",0xaa3da768,
               "fs_u3_s.l1",0xe8a0b2d1,
               "fs_u4_s.l1",0xa5de69f4,
               "fs_u5_s.l1",0x74b4d495,
               "fs_u6_s.l1",0x3c7f7a04,
               "fs_u7_s.l1",0xf32b9271,
               "fs_u8_s.l1",0xa7aafa3e,
               "fs_u9_s.l1",0x0a6664fb)
WPC_ROMEND

WPC_GAMEDEF(fs,lx5,"FS - The Flintstones Lx-5",1994,"Williams",wpc_mSecurityS,0)

/*--------------
/ Fish Tales
/---------------*/
static wpc_tGameData ftGameData = {
  WPC_GENFLIPTRON,
  {0}, /* sNO */
  0,0,0,
  FLIP_SW(FLIP_L) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(ft,3/*?*/)

WPC_ROMSTART(ft,l5,"fshtl_5.rom",0x80000,0x88847775)
WPCS_SOUNDROM8("ft_u18.l1",0x48d2760a)
WPC_ROMEND

WPC_GAMEDEF(ft,l5,"FT - Fish Tales L-5",1992, "Williams",wpc_mFliptronS,0)

/*--------------
/ Gilligans Island
/---------------*/
static wpc_tGameData giGameData = {
  WPC_GENDMD,
  {0}, /* sNO */
  0,0,0,
  FLIP_SWNO(12,11),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(gi,3/*?*/)

WPC_ROMSTART(gi,l9,"gilli_l9.rom",0x40000,0xaf07a757)
WPCS_SOUNDROM222("gi_u18.l2",0xea53e196,
                 "gi_u15.l2",0xf8241dc9,
                 "gi_u14.l2",0x0e7a4140)
WPC_ROMEND

WPC_GAMEDEF(gi,l9,"GI - Gilligan's Island L-9",1991, "Bally",wpc_mDMDS,0)

/*--------------
/ The Getaway
/---------------*/
static wpc_tGameData gwGameData = {
  WPC_GENFLIPTRON,
  {0}, /* sNO */
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_UR) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(gw,3/*?*/)

WPC_ROMSTART(gw,l5,"getaw_l5.rom",0x80000,0xb97f3d62) WPC_ROMEND

WPC_GAMEDEF(gw,l5,"GW - The Getaway: High Speed II L-5",1992, "Williams",wpc_mFliptron,GAME_NO_SOUND)

/*===========
/  Harley Davidson
/============*/
static wpc_tGameData hdGameData = {
  WPC_GENALPHA,
  {0}, /* no serial number */
  0,0,0,
  FLIP_SWNO(12,11),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(hd,3/*?*/)

WPC_ROMSTART(hd,l3,"harly_l3.rom",0x020000,0x65f2e0b4) WPC_ROMEND

WPC_GAMEDEF(hd,l3,"HD - Harley Davidson L-3",1991,"Bally",wpc_mAlpha,GAME_NO_SOUND)


/*--------------
/ Hurricane
/---------------*/
static wpc_tGameData hurrGameData = {
  WPC_GENFLIPTRON, /* ?? */
  {0}, /* sNO */
  0,0,0,
  FLIP_SWNO(12,11),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(hurr,3/*?*/)

WPC_ROMSTART(hurr,l2,"hurcnl_2.rom",0x40000,0xfda6155f)
WPCS_SOUNDROM222("u18.pp",0x63944b37,
                 "u15.pp",0x93d02c62,
                 "u14.pp",0x51c82899)
WPC_ROMEND

WPC_GAMEDEF(hurr,l2,"Hurricane L-2",1991, "Williams",wpc_mFliptronS,0)

/*--------------
/ Indianapolis 500
/---------------*/
static wpc_tGameData i500GameData = {
  WPC_GENSECURITY,
  "526 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x1f, 0xe0, 0x27, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(i500,3/*?*/)

WPC_ROMSTART(i500,11r,"indy1_1r.rom",0x80000,0xec385bf5) WPC_ROMEND

WPC_GAMEDEF(i500,11r,"I500 - Indianapolis 500 1.1r",1995,"Bally",wpc_mSecurity,GAME_NO_SOUND)

/*--------------
/ Jack*Bot
/---------------*/
static wpc_tGameData jbGameData = {
  WPC_GEN95,
  "551 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0xe0, 0x00, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(jb,3/*?*/)

WPC_ROMSTART(jb,10r,"jack1_0r.rom",0x80000,0x0e1a900a) WPC_ROMEND

WPC_GAMEDEF(jb,10r,"JB - Jack*Bot 1.0r",1995,"Williams",wpc_mSecurity,GAME_NO_SOUND)

/*--------------
/ Judge dredd
/---------------*/
static wpc_tGameData jdGameData = {
  WPC_GENDCS,
  {0}, /* sNO */
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_U),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x02, 0x00, 0xf8, 0x6e, 0x3e, 0x7f, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(jd,3/*?*/)

WPC_ROMSTART(jd,l7,"jdrd_l7.rom",0x80000,0x87b2a5c3) WPC_ROMEND

WPC_GAMEDEF(jd,l7,"JD - Judge Dredd L-7",1993,"Bally",wpc_mDCS,GAME_NO_SOUND)

/*--------------
/ Johnny Mnemonic
/---------------*/
static wpc_tGameData jmGameData = {
  WPC_GENSECURITY,
  "542 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x3f, 0x00, 0x00, 0x00, 0x78, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(jm,3/*?*/)

WPC_ROMSTART(jm,12r,"john1_2r.rom",0x80000,0xfff07398)
DCS_SOUNDROM7x("jm_u2_s.1_0",0x4aeeff3d,
               "jm_u3_s.1_0",0x9bf7bc43,
               "jm_u4_s.1_0",0x2e044582,
               "jm_u5_s.1_0",0x50cc06a7,
               "jm_u6_s.1_0",0xbfc94707,
               "jm_u7_s.1_0",0x9d4d9e9d,
               "jm_u8_s.1_0",0xfc7af6c0)
WPC_ROMEND

WPC_GAMEDEF(jm,12r,"JM - Johnny Mnemonic 1.2r",1995,"Williams",wpc_mSecurityS,0)

/*----------------
/ Junk Yard
/-----------------*/
static wpc_tGameData jyGameData = {
  WPC_GEN95,
  "552 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x7f, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(jy,3/*?*/)

WPC_ROMSTART(jy,12,"jy_g11.1_2",0x80000,0x16fb4bb3)
DCS_SOUNDROM4xm("jy_s2.rom",0x1a1bc2ca,
                "jy_s3.rom",0x0fc36a8e,
                "jy_s4.rom",0x0aebcd77,
                "jy_s5.rom",0xf18ad10b)
WPC_ROMEND

WPC_GAMEDEF(jy,12,"JY - Junk Yard 1.2",1996,"Williams",wpc_m95S,0)

/*----------------
/ NBA Fastbreak
/-----------------*/
static wpc_tGameData nbafGameData = {
  WPC_GEN95,
  "553 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(nbaf,3/*?*/)

WPC_ROMSTART(nbaf,31a,"fb_g11.3_1",0x80000,0xacd84ec2)
DCS_SOUNDROM5xm("fb-s2.1_0",0x32f42a82,
                "fb-s3.1_0",0x033aa54a,
                "fb-s4.1_0",0x6965a7c5,
                "fb-s5.1_0",0xdb50b79a,
                "fb-s6.1_0",0xf1633371)
WPC_ROMEND
WPC_ROMSTART(nbaf,31,"fb_g11.3_1",0x80000,0xacd84ec2)
DCS_SOUNDROM5m("fb-s2.3_0",0x4594abd3,
               "fb-s3.1_0",0x033aa54a,
               "fb-s4.1_0",0x6965a7c5,
               "fb-s5.1_0",0xdb50b79a,
               "fb-s6.1_0",0xf1633371)
WPC_ROMEND

WPC_GAMEDEF(nbaf,31,"NBA Fastbreak 3.1 (S3.0)",1997,"Bally",wpc_m95S,0)
WPC_CLONEDEF(nbaf,31a,31,"NBA Fastbreak 3.1 (S1.0)",1997,"Bally",wpc_m95S,0)

/*--------------
/ No Fear: Dangerous Sports
/---------------*/
static wpc_tGameData nfGameData = {
  WPC_GENSECURITY,
  "525 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0xff, 0xe3, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(nf,3/*?*/)

WPC_ROMSTART(nf,23x,"nofe2_3x.rom",0x80000,0xd853650b) WPC_ROMEND

WPC_GAMEDEF(nf,23x,"NF - No Fear: Dangerous Sports 2.3x",1995,"Williams",wpc_mSecurity,GAME_NO_SOUND)

/*----------------
/ No Good Gofers
/-----------------*/
static wpc_tGameData nggGameData = {
  WPC_GEN95,
  "561 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0xff, 0x3b, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(ngg,3/*?*/)

WPC_ROMSTART(ngg,13,"go_g11.1_3",0x80000,0x64e73117) WPC_ROMEND

WPC_GAMEDEF(ngg,13,"NGG - No Good Gofers 1.3",1997,"Williams",wpc_m95,GAME_NO_SOUND)

/*--------------
/ Popeye
/---------------*/
static wpc_tGameData popGameData = {
  WPC_GENDCS,
  {0}, /* sNO */
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_U),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x7f, 0xe0, 0x7f, 0x00, 0x00, 0x07, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(pop,3/*?*/)

WPC_ROMSTART(pop,lx5,"peye_lx5.rom",0x80000,0xee1f7a67) 
DCS_SOUNDROM6x(	"popsndl2.u2",0x00590f2d,
		"popsndl2.u3",0x87032b27,
		"popsndl2.u4",0xb0808aa8,
		"popsndl2.u5",0x3662206b,
		"popsndl2.u6",0x84a5f317,
		"popsndl2.u7",0xb8fde2c7)
WPC_ROMEND

WPC_GAMEDEF(pop,lx5,"Popeye Saves the Earth Lx-5",1994,"Bally",wpc_mDCSS,0)

/*--------------
/ Party Zone
/---------------*/
static wpc_tGameData pzGameData = {
  WPC_GENFLIPTRON,
  {0}, /* sNO */
  0,0,0,
  FLIP_SW(FLIP_L) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(pz,3/*?*/)

WPC_ROMSTART(pz,f4,"pzonef_4.rom",0x40000,0x041d7d15) 
WPCS_SOUNDROM222("pz_u18.l1",0xa269b7c4,
                 "pz_u15.l1",0x168bcc52,
                 "pz_u14.l1",0x596d9cc2)
WPC_ROMEND

WPC_GAMEDEF(pz,f4,"PZ - Party Zone F-4",1991,"Bally",wpc_mFliptronS,0)

/*--------------
/ Red and Ted's Road Show
/---------------*/
static wpc_tGameData rsGameData = {
  WPC_GENSECURITY,
  "524 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_U), /* actually 3 left flippers */
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x12, 0x00, 0x00, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(rs,3/*?*/)

WPC_ROMSTART(rs,l6,"rshw_l6.rom",0x80000,0x3986d402) WPC_ROMEND

WPC_GAMEDEF(rs,l6,"RS - Red and Ted's Road Show",1994,"Williams",wpc_mSecurity,GAME_NO_SOUND)

/*----------------
/  Safe Cracker
/-----------------*/
static wpc_tGameData scGameData = {
  WPC_GEN95,
  "903 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_UR) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x7f, 0x02, 0xe0, 0x3f, 0x3f, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(sc,3/*?*/)

WPC_ROMSTART(sc,18,"safe_18g.rom",0x80000,0xaeb4b669) 
DCS_SOUNDROM3m(	"safsnds2.rom",0x20e14c63,
		"safsnds3.rom",0x99e318e7,
		"safsnds4.rom",0x9c8a23eb)
WPC_ROMEND

WPC_GAMEDEF(sc,18,"SC - Safe Cracker 1.8",1996,"Bally",wpc_m95S,0)

/*----------------
/ Scared Stiff
/-----------------*/
static wpc_tGameData ssGameData = {
  WPC_GEN95,
  "548 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x02, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(ss,3/*?*/)

WPC_ROMSTART(ss,15,"ss_g11.1_5",0x80000,0x5de8d0a0)
DCS_SOUNDROM3m("sssnd_11.s2",0x1b080a72,
               "sssnd_11.s3",0xc4f2e08a,
               "sssnd_11.s4",0x258b0a27)
WPC_ROMEND
WPC_GAMEDEF(ss,15,"SS - Scared Stiff 1.5",1996,"Bally",wpc_m95S,0)

/*----------------
/ Tales of the Arabian Night
/-----------------*/
static wpc_tGameData totanGameData = {
  WPC_GEN95,
  "547 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(totan,3/*?*/)

WPC_ROMSTART(totan,14,"an_g11.1_4",0x80000,0x54db749e)
DCS_SOUNDROM4mx("ans2v1_1.rom",0x0d023f90,
                "ans3v1_0.rom",0x3f677813,
                "ans4v1_0.rom",0xc26dff5f,
                "ans5v1_0.rom",0x32ca1602)
WPC_ROMEND

WPC_GAMEDEF(totan,14,"TOTAN - Tales of the Arabian Nights 1.4",1996,"Williams",wpc_m95S,0)

/*----------------
/ WHO Dunnit
/-----------------*/
static wpc_tGameData wdGameData = {
  WPC_GEN95,
  "544 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x02, 0x10, 0x7f, 0xcf, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(wd,3/*?*/)

WPC_ROMSTART(wd,12,"whod1_2.rom",0x80000,0xd49be363)
DCS_SOUNDROM6x("wdu2_10.rom",0x2fd534be,
               "wdu3_10.rom",0xbe9b312f,
               "wdu4_10.rom",0x00000000,
               "wdu5_10.rom",0x00000000,
               "wdu6_10.rom",0x00000000,
               "wdu7_10.rom",0x00000000)
WPC_ROMEND

WPC_GAMEDEF(wd,12,"W?D - WHO Dunnit 1.2",1995,"Bally",wpc_m95S,GAME_NO_SOUND)

/*--------------
/ White Water
/---------------*/
static wpc_tGameData wwGameData = {
  WPC_GENFLIPTRON,
  {0}, /* sNO */
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_UR) | FLIP_SOL(FLIP_L | FLIP_UR),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xbf, 0x00, 0x60, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(ww,3/*?*/)

WPC_ROMSTART(ww,l5,"wwatr_l5.rom",0x80000,0x4eb1d233)
WPCS_SOUNDROM248("ww_u18.l1",0x6f483215,
                 "ww_u15.l1",0xfe1ae71b,
                 "ww_u14.l1",0xf3faa427)
WPC_ROMEND

WPC_GAMEDEF(ww,l5,"WW - White Water L-5",1993,"Williams",wpc_mFliptronS,0)

/*--------------------
/ Slugfest baseball
/--------------------*/
static wpc_tGameData sfGameData = {
  WPC_GENDMD,
  {0}, /* sNO */
  0,0,0,
  0,
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(sf,0/*?*/)

WPC_ROMSTART(sf,l1,"sf_u6.l1",0x40000,0xada93967)
WPCS_SOUNDROM222("sf_u18.l1",0x78092c83,
                 "sf_u15.l1",0xadcaeaa1,
                 "sf_u14.l1",0xb830b419)
WPC_ROMEND

WPC_GAMEDEF(sf,l1,"SF - SlugFest (Baseball game)",1991,"Williams",wpc_mDMDS,0)

/*-------------
/ Ticket Tac Toe
/--------------*/
static wpc_tGameData tttGameData = {
  WPC_GEN95,
  "905  123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x3f, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(ttt,0/*?*/)

WPC_ROMSTART(ttt,10,"tikt1_0.rom",0x80000,0xbf1d0382) WPC_ROMEND

WPC_GAMEDEF(ttt,10,"Ticket Tac Toe",1996,"Williams",wpc_m95,GAME_NO_SOUND)

/*-----------------------------
/ League Champ (Shuffle Alley)
/------------------------------*/
static wpc_tGameData lcGameData = {
  WPC_GENFLIPTRON,
  {0}, /* sNO */
  0,0,0,
  0,
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(lc,0/*?*/)

WPC_ROMSTART(lc,11,"lchmp1_1.rom",0x80000,0x60ab944c) WPC_ROMEND

WPC_GAMEDEF(lc,11,"LC - League Champ (Shuffle Alley)","????","Bally",wpc_mFliptron,GAME_NO_SOUND)

/*--------------
/ Test Fixture DMD generation
/---------------*/
static wpc_tGameData tfdmdGameData = {
  WPC_GENDMD,
  {0}, /* sNO */
  0,0,0,
  FLIP_SWNO(12,11),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(tfdmd,0)

WPC_ROMSTART(tfdmd,l3,"u6_l3.rom",0x20000,0xbd43e28c) WPC_ROMEND

WPC_GAMEDEF(tfdmd,l3,"Test Fixture DMD",1991,"Bally",wpc_mDMD,GAME_NO_SOUND)

/*--------------
/ Test Fixture Security generation
/---------------*/
static wpc_tGameData tfsGameData = {
  WPC_GENSECURITY,
  "648 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_U),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(tfs,0)

WPC_ROMSTART(tfs,12,"u6_12.rom",0x80000,0x12687d19)
DCS_SOUNDROM1x("u2_10.rom",0xd705b41e)
WPC_ROMEND

WPC_GAMEDEF(tfs,12,"Test Fixture Security 1.2",1994,"Bally",wpc_mSecurityS,0)

/*--------------
/ Test Fixture WPC95
/---------------*/
static wpc_tGameData tf95GameData = {
  WPC_GEN95,
  "648 123456 12345 123",
  0,0,0,
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_U),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(tf95,0)

WPC_ROMSTART(tf95,12,"g11_12.rom",0x80000,0x259a2b23)
DCS_SOUNDROM1m("s2_10.rom",0xceff7fe4)
WPC_ROMEND

WPC_GAMEDEF(tf95,12,"Test Fixture WPC95 1.2",1996,"Bally",wpc_m95S,0)

/*===========
/  Test Ficture Alphanumeric
/============*/
static wpc_tGameData tfaGameData = {
  WPC_GENALPHA,
  {0}, /* no serial number */
  0,0,0,
  FLIP_SWNO(12,11),
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, 0}
};
INITGAME(tfa,0)

WPC_ROMSTART(tfa,13,"u6_l3.rom",0x020000,0xbf4a37b5) WPC_ROMEND

WPC_GAMEDEF(tfa,13,"Test Fixture Alphanumeric 1.3",1990,"Bally",wpc_mAlpha,GAME_NO_SOUND)

