/***************************************************************************
 Sound Command Mode & Wave Recording Functionality (SJE 301000)
 ---------------------------------------------------------------------------
 
 Sound Command Mode allows user to trigger any valid sound commands to the
 Pinball Sound Hardware. This allows user to play any sound contained in the
 Sound Roms.

 Wave Recording allows user to save the current sound output to a file.

 ****************************************************************************/

#include "driver.h"
#include "wpcsound.h"
#include "dcs.h"
#include "wpcsim.h"
#include "wpc.h"
#include "sound/mixer.h"
#include "snd_cmd.h"

/*Local Functions*/
static void manual_sound_wpcs(void);
static void manual_sound_dcs(void);
static void handle_recording(void);
static char * get_wave_name(void);

/*Local Variables*/
static int recording=0;		/*Flag signifying if we're currently recording a wav file!*/
static char wavemsg[100]=""; 	/*Buffer for Wave Recording Message String*/
static int a1_ok=0;
static int a1_ofs=0;
static int d1_ok=0;
static int d1_ofs=0;
static int dcs_command=0;	/*Current DCS Command to play*/
static int snd_command=0;	/*Current Command to play*/
static int ynum=0;		/*Y-offset for displaying help text*/
static int kill_it=0;		/*Kill currently playing sound*/

/*-----------------------------------------
  Initialize Sound Command Variables
  ----------------------------------------*/
void snd_cmd_init()
{
recording=sound_mode=snd_command=ynum=kill_it=d1_ok=d1_ofs=dcs_command=a1_ok=a1_ofs=0;
sprintf(wavemsg," ");
}

/*-----------------------------------------
  Sound Command Mode Functionality
  ----------------------------------------*/
void manual_sound_commands(struct osd_bitmap *bitmap)
{
char str[100];
ynum=0;

/*Handle recording of wave file functionality*/
handle_recording();

/*Are we attempting to kill the currently playing sound?*/
if(kill_it)
{
	if (wpc_gameData->wpcGen < WPC_GENDCS) 
		manual_sound_wpcs();
	else
		manual_sound_dcs();
	return;
}

/*-------------------------*/
/*-- TOGGLE COMMAND MODE --*/
/*-------------------------*/
if(sound_mode==0)					/*Not in command mode*/
{
	/*Change Sound Command Mode?*/
	if(keyboard_pressed_memory_repeat(SMDCMD_MODETOGGLE,REPEATKEY))
	{
	sound_mode=1;
	/*Clear out what we drew on screen*/
	fillbitmap(bitmap,Machine->uifont->colortable[0],NULL);
	schedule_full_refresh();
	/*Halt CPU #0 Operation*/
	cpu_set_halt_line(0,1);
	}
	return;
}

if(sound_mode==1)					/*If in Command Mode*/
{
	/*Change Sound Command Mode?*/	
	if(keyboard_pressed_memory_repeat(SMDCMD_MODETOGGLE,REPEATKEY))
	{
	sound_mode=0;
	/*Clear out what we drew on screen*/
	fillbitmap(bitmap,Machine->uifont->colortable[0],NULL);
	schedule_full_refresh();
	/*Resume CPU #0 Operation*/
	cpu_set_halt_line(0,0);
	/*Clear any running commands*/
	kill_it=1;	
	return;
	}
}

/* Command ++ */
if(keyboard_pressed_memory_repeat(SMDCMD_NEXT,REPEATKEY))
{
	if(snd_command<=MAXCOMMAND-1)
	{snd_command++;}
}
/* Command -- */
if(keyboard_pressed_memory_repeat(SMDCMD_PREV,REPEATKEY))
{
	if(snd_command>=1)
	{snd_command--;}
}
/* Command = 0*/
if(keyboard_pressed_memory_repeat(SMDCMD_ZERO,REPEATKEY))
{snd_command = 0;}

/*Show current command #*/
sprintf(str,"* * * SOUND COMMAND MODE * * *");
wpc_textOut(str, strlen(str), SND_XROW,ynum, BLACK);
sprintf(str,"Sound Command=%04d",snd_command);
wpc_textOut(str, strlen(str), SND_XROW+30,ynum+=20, BLACK);

/*Play a sound*/
if (wpc_gameData->wpcGen < WPC_GENDCS) 
	manual_sound_wpcs();
else
	manual_sound_dcs();

/*Show keys to use*/
sprintf(str,"- - Keys - -");
wpc_textOut(str, strlen(str), SND_XROW+40,ynum+=20, BLACK);
sprintf(str,"Arrows (L&R): Command #");
wpc_textOut(str, strlen(str), SND_XROW+30,ynum+=10, BLACK);
if (wpc_gameData->wpcGen >= WPC_GENDCS) 
	{
	sprintf(str,"Arrows (U&D): DCS Bank #");
	wpc_textOut(str, strlen(str), SND_XROW+30,ynum+=10, BLACK);
	}
sprintf(str,"Delete: Reset to 0");
wpc_textOut(str, strlen(str), SND_XROW+30,ynum+=10, BLACK);
sprintf(str,"Spacebar: Trigger Command");
wpc_textOut(str, strlen(str), SND_XROW+30,ynum+=10, BLACK);
}

/*-----------------------------------------
  Process a Sound Command - DCS Hardware
  ----------------------------------------*/
static void manual_sound_dcs(void)
{
int dcs_commands[] = {3,227,3,230,3,229,3,227,3,230,3,229};
char str[100];
if(!kill_it)
	{
		/* Command ++ */
		if(keyboard_pressed_memory_repeat(SMDCMD_UP,REPEATKEY))
			{
			if(dcs_command<=MAXCOMMAND-1)
			{dcs_command++;}
			}
		/* Command -- */
		if(keyboard_pressed_memory_repeat(SMDCMD_DOWN,REPEATKEY))
			{
			if(dcs_command>=1)
			{dcs_command--;}
			}
	}
else
	{
		d1_ok++;
		snd_command=0;
	}

if (keyboard_pressed(SMDCMD_PLAY))
  {
	logerror("DCS: Pressed Key\n");
	while(keyboard_pressed(SMDCMD_PLAY));
		d1_ok=1;
  } 
else 
	{
		if (d1_ok>0)
		  d1_ok++;
	}

if (d1_ok>3)
	{
	int comm = dcs_commands[d1_ofs];
	if(d1_ofs==DCS_COMMS-2) 
		comm = dcs_command;
	if(d1_ofs==DCS_COMMS-1) 
		comm = snd_command;
	dcs_data_w(0, comm);
	logerror("DCS: Sending %d\n",comm);
	d1_ofs++;
	d1_ok=1;

	if (d1_ofs==DCS_COMMS) // all commands sent ?
		{
		   d1_ok=0;
		   d1_ofs=0;
		   if(kill_it) 
		   {
			   kill_it=0;
			   return;
		   }
		}
	}
if(!kill_it)
	{
	sprintf(str,"DCS Bank #=%04d",dcs_command);
	wpc_textOut(str, strlen(str), SND_XROW+30,ynum+=10, BLACK);
	}
}

/*-----------------------------------------
  Process a Sound Command - NON-DCS Hardware
  ----------------------------------------*/
static void manual_sound_wpcs(void)
{
int wcs_commands[] = {126,125,127,122};
//int wcs_commands[] = {1,122,13,244};

if(kill_it)
	{
		a1_ok++;
		snd_command=0;
	}

if (keyboard_pressed(SMDCMD_PLAY))
  {
	while(keyboard_pressed(SMDCMD_PLAY));
		a1_ok=1;
  } 
else 
  {
	if (a1_ok>0)
		  a1_ok++;
  }

if (a1_ok>3)
  {
	int comm = wcs_commands[a1_ofs];
	if(a1_ofs==WPCS_COMMS-1)
		comm = snd_command;
	wpcs_data_w(0, comm);
	a1_ofs++;
	a1_ok=1;

	if (a1_ofs==WPCS_COMMS) // all commands sent ?
		{
		   a1_ok=0;
		   a1_ofs=0;
		   if(kill_it) 
		   {
			   kill_it=0;
			   return;
		   }
		}
  }
}

/*-----------------------------------------
  Handle Wave Recording
  ----------------------------------------*/
static void handle_recording(void)
{
/*If Wave Toggle Key pressed, toggle Recording of Wave File*/
if(keyboard_pressed_memory_repeat(SMDCMD_RECORDTOGGLE,REPEATKEY))
{
	char name[120];

	if(recording){
		wave_record_stop();
		recording=0;
	}
	else{
		recording=1;
		strcpy(name,get_wave_name());
		logerror("Using %s for wave filename!\n",name);
		if(wave_record_start(name)>0)
			sprintf(wavemsg,"Recording Wave File...");		
		else
			sprintf(wavemsg,"Unable to Record Wave File!!!");			
	}
}
/*Display if we're recording!*/
if(recording)
	wpc_textOut(wavemsg, strlen(wavemsg), 30,100, BLACK);
else
	wpc_textOut("", 50, 30,100, BLACK);
return;
}


int waverec=0;

static char * get_wave_name()
{
	char name[20];
	char *ret;

	/* avoid overwriting existing files */
	/* first of all try with "gamename.wav" */
	sprintf(name,"%.8s", Machine->gamedrv->name);
	if (osd_faccess(name,OSD_FILETYPE_WAVEFILE))
	{
		do
		{
		  /* otherwise use "nameNNNN.wav" */
		  sprintf(name,"%.4s%04d",Machine->gamedrv->name,waverec++);
		} while (osd_faccess(name, OSD_FILETYPE_WAVEFILE));
	}
	ret = name;
	return ret;
}
