/********************************************************************************************
 Williams Star Trek Next Generation (1994) Pinball Simulator

 by Steve Ellenoff (sellenoff@hotmail.com)
 Nov.3, 2000

 Known Issues/Problems with Simulator:
 -None at the moment.

 Notes:
 ------
*********************************************************************************************************/


/*-------------------------------------------------------------------------------------------------
  Keys for Star Trek Next Generation Simulator:
  ----------------------------
    +R		L/R Ramp
(SPACE)+R	Middle Ramp
    +I		L/R Inlane
    +-		L/R Slingshot
    +L		L/R Loop
    +O		L/R Outlane
   ASD		Left Drop Targets (1-3)
   FGH		Right Drop Targets (1-3)
   JKL		Center Drop Targets (1-3)
	 W		Q Drop Target
	 /		Center Hole
	 .		Left Hole
	ZX		Time Rift Targets (1-2)
   CVB		Rollovers (1-3)
	NM,		Jet Bumpers (1-3)
     Q		SDTM (Drain Ball)
-------------------------------------------------------------------------------------------------*/

#include "driver.h"
#include "wpc.h"
#include "wpcsim.h"
#include "dcs.h"

/*------------------
/  Local functions
/-------------------*/
/*-- local state/ball handling functions --*/
static int  sttng_handleBallState(wpcsim_tBallStatus *ball, int *inports);
static void sttng_handleMech(int *inports);
static void sttng_drawMech(unsigned char **line);
static void init_sttng(void);
static int sttng_getSol(int solNo);

/*-----------------------
  local static variables
 ------------------------*/
static struct {
  int TdroptargetPos;    /* Top Drop Target Position */
  int LgunPos;			 /* Left Gun Position  */
  int LgunDir;		     /* Left Current Direction of Gun Movement*/
  int LgunLDir;			 /* Left Last direction of Gun Movement*/
  int RgunPos;			 /* Right Gun Position  */
  int RgunDir;		     /* Right Current Direction of Gun Movement*/
  int RgunLDir;			 /* Right Last direction of Gun Movement*/
  int UTDivPos;			 /* Keep Track of Under Playfield Top Diverter*/
  int UBDivPos;			 /* Keep Track of Under Playfield Bott Diverter*/
  } locals;

/*--------------------------
/ Game specific input ports
/---------------------------
   Here we define for WPCMAME which Keyboard Presses it should look for, and process!
--------------------------------------------------------------------------------------------------*/
WPC_INPUT_PORTS_START(sttng,6)
  PORT_START /* 0 */
    WPCPORT_BIT(   0x0001,"Left Qualifier",  KEYCODE_LSHIFT)
    WPCPORT_BIT(   0x0002,"Right Qualifier", KEYCODE_RSHIFT)
    WPCPORT_BITIMP(0x0004,"L/R Ramp",        KEYCODE_R)
    WPCPORT_BITIMP(0x0008,"L/R Outlane",     KEYCODE_O)
    WPCPORT_BITIMP(0x0010,"L/R Loop",        KEYCODE_L)
    WPCPORT_BIT(   0x0040,"L/R Slingshot",   KEYCODE_MINUS)
    WPCPORT_BIT(   0x0080,"L/R Inlane",      KEYCODE_I)
    WPCPORT_BITIMP(0x0100,"L.Drop Target 1", KEYCODE_A)
    WPCPORT_BITIMP(0x0200,"L.Drop Target 2", KEYCODE_S)
    WPCPORT_BITIMP(0x0400,"L.Drop Target 3", KEYCODE_D)
    WPCPORT_BITIMP(0x0800,"R.Drop Target 1", KEYCODE_F)
    WPCPORT_BIT(   0x1000,"R.Drop Target 2", KEYCODE_G)
    WPCPORT_BIT(   0x2000,"R.Drop Target 3", KEYCODE_H)
    WPCPORT_BITIMP(0x4000,"C.Drop Target 1", KEYCODE_J)
    WPCPORT_BIT(   0x8000,"C.Drop Target 1", KEYCODE_K)
  PORT_START /* 1 */
    WPCPORT_BIT(   0x0001,"C.Drop Target 1", KEYCODE_L)
    WPCPORT_BIT(   0x0002,"Q Drop Target",   KEYCODE_W)
    WPCPORT_BIT(   0x0004,"Center Hole",     KEYCODE_SLASH)
    WPCPORT_BIT(   0x0008,"Left Hole",       KEYCODE_STOP)
    WPCPORT_BIT(   0x0010,"Time Rift 1",     KEYCODE_Z)
    WPCPORT_BIT(   0x0020,"Time Rift 2",     KEYCODE_X)
    WPCPORT_BIT(   0x0040,"Rollover 1",      KEYCODE_C)
    WPCPORT_BIT(   0x0100,"Rollover 2",      KEYCODE_V)
    WPCPORT_BIT(   0x0200,"Rollover 3",      KEYCODE_B)
    WPCPORT_BIT(   0x0400,"Jet 1",           KEYCODE_N)
    WPCPORT_BIT(   0x0800,"Jet 2",           KEYCODE_M)
    WPCPORT_BIT(   0x1000,"Jet 3",           KEYCODE_COMMA)
    WPCPORT_BITIMP(0x2000,"Drain",           KEYCODE_Q)
    WPCPORT_BIT(   0x4000,"Buy-In",			 KEYCODE_2)
//	  WPCPORT_BIT(   0x8000,"Jet Bumper Lane", KEYCODE_Y)
WPC_INPUT_PORTS_END

/*-------------------
/ Switch definitions
/--------------------*/

#define swBuyIn			11
#define swFire			12
#define swStart      	13
#define swTilt       	14
#define swLOut		 	15
#define swLIn			16
#define swRIn			17
#define swROut			18

#define swSlamTilt   	21
#define swCoinDoor   	22
#define swCRampExit    	23
#define swNotUsed		24	
#define swRRampEnt		25
#define	swCBankL		26
#define	swCBankC		27
#define swCBankR		28

/*All switchs in this column are optos*/
#define	swBorgLock		31
#define	swULGun2		32
#define	swURGun2		33
#define	swRGunShooter	34
#define	swULLock2		35
#define	swULGun1		36
#define	swURGun1		37
#define	swLGunShooter	38

/*All switchs in this column are optos*/
#define swULLock1     	41
#define swULLock3		42
#define swULLock4		43
#define swLOuterLoop	44
#define swUTopHole		45
#define swULeftHole		46
#define swUBorgHole  	47
#define swBorgEntry		48

#define swLBankTop		51
#define swLBankMid		52
#define swLBankBott    	53
#define swRBankTop		54
#define swRBankMid		55
#define swRBankBott		56	
#define swTDropTarget	57
#define swROuterLoop	58	

/*All switchs in this column are optos -except last switch!-*/
#define swTrough1		61
#define swTrough2    	62
#define swTrough3    	63
#define swTrough4		64
#define swTrough5		65
#define swTrough6		66
#define swTroughUp		67
#define swShooter      	68

#define swJet1			71
#define swJet2	    	72
#define swJet3		  	73
#define swRSling    	74
#define swLSling		75
#define swMultL			76
#define swMultC       	77
#define swMultR       	78

#define swTime			81
#define swRift	    	82
#define swLRampExit    	83
#define swQ			  	84
#define swLShuttle		85
#define swRShuttle		86
#define swRRampExit    	87
#define swLRampEnt     	88

/*Since normal switches end at 88, we need custom ones!*/

/*91 Not Used*/
#define swLGunMark    	WPC_CUSTSWNO(1,2)		//92
/*93 Not Used*/
#define swRGunHome		WPC_CUSTSWNO(1,5)		//95
#define swRGunMark    	WPC_CUSTSWNO(1,6)		//96
#define swLGunHome		WPC_CUSTSWNO(1,7)		//97
/*98 Not Used*/

/*Spinner = F7*/
#define swSpinner		swF8


/*---------------------
/ Solenoid definitions
/----------------------*/
#define sLGunKicker    	1
#define sRGunKicker    	2
#define sLGunPopper  	3
#define sRGunPopper		4
#define sLPopper		5
#define sPlunger		6
#define sKnocker       	7
#define sKickBack   	8
#define sLSling      	9
#define sRSling      	10
#define sTrough			11
#define sJet1         	12
#define sJet2        	13
#define sJet3        	14
#define sTopDiverter	15
#define sBorgKicker		16
#define sLGunMotor    	17
#define sRGunMotor    	18
/*19 Not Used*/

/*We need custom solenoids, since regular ones end at #36*/
#define sUDiverterTop  	WPC_CUSTSOLNO(1)	//37
#define sUDiverterBott 	WPC_CUSTSOLNO(2)	//38
#define sTopDropUp     	WPC_CUSTSOLNO(3)	//39
#define sTopDropDown   	WPC_CUSTSOLNO(4)	//40


#define GUN_HOME    0	    /*Beginning Position*/
#define GUN_MARK    (18*5)  /* Where is Mark Switch Triggered. Roughly 1/3 way between home & end*/ 
#define GUN_END		(40*5)  /*# of Times to increment rotation until we stop and turnaround!*/
#define GUN_LEFT    0
#define GUN_RIGHT   1
#define DT_UP		0
#define DT_DOWN		1
#define DIV_OPEN	0
#define DIV_CLOSED	1


/*---------------------
/  Ball state handling
/----------------------*/

/* ---------------------------------------------------------------------------------
   The following variables are used to refer to the state array!
   These vars *must* be in the *exact* *same* *order* as each stateDef array entry! 
   -----------------------------------------------------------------------------------------*/
enum {stTrough1=WPCSIM_FIRSTSTATE, stTrough2, stTrough3, stTrough4, stTrough5, stTrough6, stTrough,
	  stShooter, stLaunchWire, stDrain,
	  stLRampEnt,stLRampExit, stCRampExit, stRRampEnt, stRRampExit,
	  stLOut, stROut, stLIn, stRIn, stLLoopUp1, stSpinner, stLLoopUp2, stRLoopDn, stRLoopUp, 
	  stTDropTarget, stTDropTarget2, stMultL, stMultC, stMultR,
      stJet1, stJet2, stJet3,
      stTopHole, stCenterHole, stLeftHole,  stTopDiverter, stBottDiverter,
	  stTLPopper4,stTLPopper3,stTLPopper2, stTLPopper1,
	  stBLPopper, stBLPopper2,stRPopper2, stRPopper, stLGunLoaded, stRGunLoaded
      };

/********************************************************************************************************
   The following is a list of all possible game states..... 
   We start defining our own states at element #3, since 'Not Installed', 'Moving', and 'PF'
   are always defined the same!!

   Any state with all zeros, like the following example must be handled in the Handle Ball State function!
   {"State Name",    1,0,    0,       0, 0}, 

   Fields for each array element
   ----------------------------- 
   #1) Name to display on screen
   #2) Switch down time (minimum)
   #3) Switch to be triggered while in this state
   #4) Solenoid used to get out of state (0=leave ASAP) also: Balls can not enter state while solenoid is active
   #5) State following this one
   #6) Time between this state and next
   #7) If this solenoid is active go to altstate (below)
   #8) alternative state
   #9) State flags
	STSWKEEP:        Leave switch on (e.g. drop targets)
	STSHOOT:         Set ballspeed to plunger speed
	STIGNORESOL:     Always call statehandler regardless of solenoid state
	STNOTEXCL:       More than one ball can be in this sate
	STNOTBALL(type): Specified ball type does not affect switch
	STSPINNER        Switch is a spinner
*******************************************************************************************************/
static wpcsim_tState sttng_stateDef[] = {
  {"Not Installed",    0,0,           0,        stDrain,     0,0,0,WPCSIM_STNOTEXCL},
  {"Moving"},
  {"PF",               0,0,           0,        0,           0,0,0,WPCSIM_STNOTEXCL},
 
  {"Trough 1",			1,swTrough1,	0,		  stTrough2,   1},
  {"Trough 2",			1,swTrough2,	0,        stTrough3,   1},
  {"Trough 3",			1,swTrough3,	0,        stTrough4,   1},
  {"Trough 4",			1,swTrough4,	0,        stTrough5,   1},
  {"Trough 5",			1,swTrough5,	0,        stTrough6,   1},
  {"Trough 6",			1,swTrough6,	0,        stTrough,    1},
  {"Trough Up",			1,swTroughUp,   sTrough,  stShooter,   1},

  {"Shooter",		    1,swShooter,   sPlunger, stLaunchWire, 1},
  {"Launch Wire",		1,0,	       0,		 stRLoopUp,   10},
  {"Drain",				1,0,           0,        stTrough1,   0,0,0,WPCSIM_STNOTEXCL},
  
  {"L. Ramp Enter",		1,swLRampEnt,   0,       stLRampExit,   5},
  {"L. Ramp Exit",		1,swLRampExit,  0,       stRIn,        10},
  {"C. Ramp Exit",		1,swCRampExit,  0,       stLIn,        10},
  {"R. Ramp Enter",		1,swRRampEnt,   0,       stRRampExit,   5},
  {"R. Ramp Exit",		1,swRRampExit,  0,       stRIn,        10},

  {"Left Outlane",      1,swLOut,       0,        stDrain,   10, sKickBack, stFree},
  {"Right Outlane",     1,swROut,       0,        stDrain,   10},
  {"Right Inlane",     1,swRIn,         0,        stFree,      5},
  {"Left  Inlane",     1,swLIn,         0,        stFree,      5},

  {"Left Loop",        1,0,				0,        stSpinner,  3},
  {"Spinner",          1,swSpinner,		0,		  stLLoopUp2,   5, 0,0,WPCSIM_STSPINNER},
  {"Left Loop",        1,swLOuterLoop,	0,		  stRLoopDn,  10},
  {"Right Loop",       1,swROuterLoop,	0,        stFree,	  10},		/*Right Loop Coming Down*/
  {"Right Loop",       1,swROuterLoop,	0,        stTDropTarget,10},	/*Right Loop Going Up */
  
  {"Top Drop Target",	1,0, 0, 0, 0},
  {"Top Drop Target",	1,swTDropTarget, 0,		  stMultL, 5},
  {"Rollover 1",		1,swMultL,		0,		  stJet2,  5},
  {"Rollover 2",		1,swMultC,		0,		  stJet3,  5},
  {"Rollover 3",		1,swMultR,		0,		  stJet1,  5},
  {"Jet Bumper 1",		1,swJet1, 0, stFree, 5},
  {"Jet Bumper 2",		1,swJet1, 0, stFree, 5},
  {"Jet Bumper 3",		1,swJet1, 0, stFree, 5},

		/*Under Bott Diverter firing, Goes to L. Gun, otherwise L.Popper
		  Under Top Diverter firing, Goes to R. Gun
		*/

  {"Top Hole",			1,swUTopHole,	0,			stLeftHole,		 1},	/*Top Holes Next passes Left hole Switch*/
  {"Center Hole",	    1,swUBorgHole,	0,			stTopDiverter,   1},	/*Center Next goes to Top Diverter*/
  {"Left Hole",			1,swULeftHole,	0,			stTopDiverter,   1},	/*Left Next goes to Top Diverter*/
//  {"U.Top Diverter",	1,0,			0,			stBottDiverter,	 1, sUDiverterTop, stRPopper2},	/*If Underside Top Diverter firing, send to Right Popper/Gun, otherwise, bottom diverted!*/
//  {"U.Bott Diverter",	1,0,			0,			stTLPopper4,	 1, sUDiverterBott, stBLPopper}, /*If Underside Bott Diverter firing, send to Left Popper/Gun, otherwise, top left popper!*/
  {"U.Top Diverter",	1,0,			0,			0,	 0},
  {"U.Bott Diverter",	1,0,			0,			0,	 0}, 

  {"Top L. Pop 4",		1, swULLock4,	0,			stTLPopper3,	 1},	/*TLPopper4*/
  {"Top L. Pop 3",		1, swULLock3,	0,			stTLPopper2,	 1},	/*TLPopper3*/
  {"Top L. Pop 2",		1, swULLock2,	0,			stTLPopper1,	 1},	/*TLPopper2*/
  {"Top L. Pop 1",		1, swULLock1,   sLPopper,	stRIn,			 1},	/*TLPopper1*/
  
  {"B. L. Pop 2",		1, swULGun2,	0,			stBLPopper2,	 1},	/*BLPopper*/
  {"B. L. Pop 1",		1, swULGun1,	sLGunPopper,stLGunLoaded,	 5},	/*BLPopper2*/
  {"R. Pop 2",			1, swURGun2,	0,				stRPopper,		1},	/*RPopper2*/
  {"R. Pop 1",			1, swURGun1,	sRGunPopper,	stRGunLoaded,	5}, /*RPopper*/
  {"L. Gun Loaded",		1, swLGunShooter, sLGunKicker,  stFree,		    2},
  {"R. Gun Loaded",		1, swRGunShooter, sRGunKicker,	stFree,		    2},
  {0}
};

static int sttng_handleBallState(wpcsim_tBallStatus *ball, int *inports) {
  switch (ball->state)
	{	
		/*-----------------
		  Top Drop Target 
		-------------------*/
		case stTDropTarget:
		/* If Drop Target Is Down.. Ball goes into Top Hole!*/
		/* Otherwise simply trigger Drop Target Switch  */
		if (locals.TdroptargetPos==DT_DOWN)
			return setState(stTopHole,25);
		else
			return setState(stTDropTarget2,25);
		break;

		/*-------------------------
		  Underfield Top Diverter
		-------------------------*/
		/*If Underside Top Diverter firing(CLOSED), send to Right Popper/Gun, otherwise, bottom diverted!*/
		case stTopDiverter:
		if (locals.UTDivPos==DIV_CLOSED)
			return setState(stRPopper2,1);
		else
			return setState(stBottDiverter,1);
		break;

		/*-------------------------
		  Underfield Top Diverter
		-------------------------*/
		
		/*If Underside Bott Diverter firing(OPEN), send to Left Popper/Gun, otherwise, top left popper!*/
		case stBottDiverter:
		if (locals.UBDivPos==DIV_OPEN)
			return setState(stBLPopper,1);
		else
			return setState(stTLPopper4,1);
		break;


  	}
  return 0;
}

/*---------------------------
/  Keyboard conversion table
/----------------------------
-------------------------------------------------------------------------------------------------- 
  Layout of inportData array
   ----------------------------
   #1) Inport number
   #2) Mask for this switch (all bits must be set for switch to activate
       Only first match will be used. Placed qualified keys first
   #3) Switch to activate or new ball state
   #4) Condition for the action to occur
                 0                  : ball in stFree state
                 switch             : ball in stFree state and switch active
                 WPCSIM_STANY       : ball can be in any state
                 WPCSIM_CUSTCOND(n) : call custom condition handler (keyCond)
--------------------------------------------------------------------------------------------------*/

static wpcsim_tInportData sttng_inportData[] = {
  {0, 0x0005, stLRampEnt},
  {0, 0x0006, stRRampEnt},
  {0, 0x0007, stCRampExit},
  {0, 0x0009, stLOut},
  {0, 0x000a, stROut},
  {0, 0x0011, stLLoopUp1},
  {0, 0x0012, stRLoopUp},
  {0, 0x0041, swLSling},
  {0, 0x0042, swRSling},
  {0, 0x0081, stLIn},
  {0, 0x0082, stRIn},
  {0, 0x0100, swLBankTop},
  {0, 0x0200, swLBankMid},
  {0, 0x0400, swLBankBott},
  {0, 0x0800, swRBankTop},
  {0, 0x1000, swRBankMid},
  {0, 0x2000, swRBankBott},
  {0, 0x4000, swCBankL},
  {0, 0x8000, swCBankC},
  {1, 0x0001, swCBankR},
  {1, 0x0002, swQ},
  {1, 0x0004, stCenterHole},
  {1, 0x0008, stLeftHole},
  {1, 0x0010, swTime},
  {1, 0x0020, swRift},
  {1, 0x0040, stMultL},
  {1, 0x0100, stMultC},
  {1, 0x0200, stMultR},
  {1, 0x0400, swJet1},
  {1, 0x0800, swJet2},
  {1, 0x1000, swJet3},
  {1, 0x2000, stDrain},
  {1, 0x4000, swBuyIn},
//  {1, 0x8000, stJetLane},
  {0}
};

/*--------------------
  Drawing information
  --------------------------------------------------------
  Code to draw the mechanical objects, and their states! 
---------------------------------------------------------*/
static void sttng_drawMech(unsigned char **line) {
  wpc_textOutf(30, 0,BLACK,"Top Drop Target: %-6s", locals.TdroptargetPos==DT_UP?"Up":"Down");
  wpc_textOutf(30, 10,BLACK,"Left Gun: %3d", locals.LgunPos);
  wpc_textOutf(30, 20,BLACK,"Right Gun: %3d", locals.RgunPos);
  wpc_textOutf(30, 30,BLACK,"UTop Div. Sol %-6s", locals.UTDivPos==DIV_OPEN?"Open":"Closed");
  wpc_textOutf(30, 40,BLACK,"UBott Div. Sol %-6s",locals.UBDivPos==DIV_OPEN?"Open":"Closed");
//wpc_textOutf(30, 40,BLACK,"Board 1 %4x", wpc_data[WPC_EXTBOARD1]);
}

/*-----------------
/  ROM definitions
/------------------*/
WPC_ROMSTART(sttng,l7,"trek_lx7.rom",0x80000,0xd439fdbb)
DCS_SOUNDROM7x("ng_u2_s.l1",0xc3bd7bf5,
               "ng_u3_s.l1",0x9456cac7,
               "ng_u4_s.l1",0x179d22a4,
               "ng_u5_s.l1",0x231a3e72,
               "ng_u6_s.l1",0xbb21377d,
               "ng_u7_s.l1",0xd81b39f0,
               "ng_u8_s.l1",0xc9fb065e)
WPC_ROMEND

/*--------------
/  Game drivers
/---------------*/
WPC_GAMEDEF(sttng,l7,"STTNG - Star Trek: The Next Generation Lx-7",1993,"Williams",wpc_mDCSS,0)

/*-----------------------
/ Simulation Definitions
/-----------------------*/
static wpcsim_tSimData sttngSimData = {
  2,    		/* 2 game specific input ports */
  sttng_stateDef,	/* Definition of all states */
  sttng_inportData,	/* Keyboard Entries */
  { stTrough6, stTrough5, stTrough4, stRPopper, stTLPopper1, stBLPopper2, stDrain }, /*Position where balls start.. Max 7 Balls Allowed*/
//  { stTrough1, stTrough2, stTrough3, stTrough4, stTrough5, stTrough6, stDrain }, /*Position where balls start.. Max 7 Balls Allowed*/
  NULL, 		/* no init */
  sttng_handleBallState,/*Function to handle ball state changes*/
  sttng_handleMech,	/*Function to handle mechanical state changes*/
  FALSE, 		/* Do not simulate manual shooter */
  NULL  		/* no custom key conditions */
};

/*----------------------
/ Game Data Information
/----------------------*/
static wpc_tGameData sttngGameData = {
  WPC_GENDCS,		/* generation */
  {0}, 			/* no serial number */
  1,			/* 1 custom switch column */
  0,			/* no custom lampcolumns */
  4,			/* 4 custom soleniods/flashers */
  FLIP_SW(FLIP_L | FLIP_U) | FLIP_SOL(FLIP_L | FLIP_UR), 	/* Which switches are the flippers */
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00}, /* inverted switches */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, swFire},
  sttng_getSol, 		/* custom solenoid handler needed */
  NULL, NULL, NULL, 	/* switch, lamp & solenoid positions */
  sttng_drawMech,	    /* Function to Draw the Status of Mechanical Objects on Screen */
  &sttngSimData		    /* Definition of Simulator Data */
};

/*---------------
/  Game handling
/----------------*/
/*-- patched memory read function to handle the 9th switch column --*/
READ_HANDLER(sttng_swRowRead) {
  if ((wpc_data[WPC_EXTBOARD1] && 0x80) > 0) /* 9th column enabled */
    return wpc_globals.swMatrix[WPC_CUSTSWCOL];
  return wpc_r(WPC_SWROWREAD);
}

static void init_sttng(void) {
  wpc_gameData = &sttngGameData;
  install_mem_read_handler(WPC_CPUNO, WPC_SWROWREAD+WPC_BASE, WPC_SWROWREAD+WPC_BASE,
                         sttng_swRowRead);

}

static int sttng_getSol(int solNo) {
  if (solNo == sUDiverterTop)			//37
    return (wpc_data[WPC_EXTBOARD1] & 0x01) > 0;		//*Normally open, closed when solenoid firing!!*/
//	  return locals.UTDivPos;

  if (solNo == sUDiverterBott)			//38
    return (wpc_data[WPC_EXTBOARD1] & 0x02) > 0;		//*Normally closed, open when solenoid firing!!*/
//	  return locals.UBDivPos;

  if (solNo == sTopDropUp)				//39
    return (wpc_data[WPC_EXTBOARD1] & 0x04) > 0;
  if (solNo == sTopDropDown)			//40
    return (wpc_data[WPC_EXTBOARD1] & 0x08) > 0;
  return 0;
}


static int utdivcnt=0;
static int ubdivcnt=0;

static void sttng_handleMech(int *inports) {

	/*Set the Status of the Underplayfield Diverters*/
	/*If Under-Top-Diverter Solenoid Fires - Set Position to Closed!
	  Reset Counter for opening it back!*/
	if(wpc_getSol(sUDiverterTop))
//	if(wpc_data[WPC_EXTBOARD1] & 0x01 > 0)
	{
		locals.UTDivPos = DIV_CLOSED;
		utdivcnt = 0;
	}
	else
    /*Solenoid not firing, if after 10 checks, it hasn't fired, we flag it as open!*/
	{
		utdivcnt++;
		if(utdivcnt>100)
		{
			utdivcnt=0;
			locals.UTDivPos = DIV_OPEN;
		}
	}

	/*If Under-Bott-Diverter Solenoid Fires - Set Position to Open!
	  Reset Counter for closing it back!*/
	if(wpc_getSol(sUDiverterBott))
//	if(wpc_data[WPC_EXTBOARD1] & 0x02 > 0)
	{
		locals.UBDivPos = DIV_OPEN;
		ubdivcnt = 0;
	}
	else
    /*Solenoid not firing, if after 10 checks, it hasn't fired, we flag it as closed!*/
	{
		ubdivcnt++;
		if(ubdivcnt>100)
		{
			ubdivcnt=0;
			locals.UBDivPos = DIV_CLOSED;
		}
	}

	/*Raise & Lower Drop Target*/
	if(wpc_getSol(sTopDropUp))
		locals.TdroptargetPos=DT_UP;
	if(wpc_getSol(sTopDropDown))
		locals.TdroptargetPos=DT_DOWN;

	/*If Drop Target is Down, switch should be active!*/
	wpc_setSw(swTDropTarget,locals.TdroptargetPos==DT_DOWN);

	wpc_setSw(swLGunHome,1);
	wpc_setSw(swRGunHome,0);
	wpc_setSw(swLGunMark,1);
	wpc_setSw(swRGunMark,0);
}
