/********************************************************************************************
 Williams Terminator 2: Judgement Day (1991) Pinball Simulator

 by Steve Ellenoff (sellenoff@hotmail.com)
 Nov.2, 2000

 Known Issues/Problems with Simulator:
 -Gun Position & Timing Totally guessed..
	but seems ok!! (Sure is hard aiming the gun when you can't see it!)
 -Skull Target does not open when it should, 
    so a hack was used... For some reason, during
    game play, the knockdown solenoid *should* fire when the Drop Target is activated but it does
    not.. So I had to put in an extra check for the switch being active to pull down the drop target
    rather than just relying on the knockdown solenoid!

 Notes:

  This game was incredibly easy to simulate. There's very little stuff going on.. No diverters,
  nothing fancy.. The only real pain was the gun. That took 80% of the time to figure out. The
  rest of the stuff are straight on shots which simply trigger switches. The ramps are straightforward!
 
 *********************************************************************************************************/


/*-------------------------------------------------------------------------------------------------
  Keys for Terminator 2 Simulator:
  ----------------------------
    +R  L/R Ramp
    +I  L/R Inlane
    +-  L/R Slingshot
    +L  L/R Loop
    +O  L/R Outlane
    WER Kickback Drop Targets(1-3)
      T Skull Drop Target
      Q SDTM (Drain Ball)
    HJK Jet Bumpers
    XCV Middle Drop Targets(1-3)
  NM,./ Left Drop Targets(1-5)
    ASD Multiplier Rollovers(1-3)
	  Z Left Lock

-------------------------------------------------------------------------------------------------*/

#include "driver.h"
#include "wpc.h"
#include "wpcsim.h"
#include "wpcsound.h"

/*------------------
/  Local functions
/-------------------*/
/*-- local state/ball handling functions --*/
static int  t2_handleBallState(wpcsim_tBallStatus *ball, int *inports);
static void t2_handleMech(int *inports);
static void t2_drawMech(unsigned char **line);
static void init_t2(void);

/*-----------------------
  local static variables
 ------------------------
   I used this locals structure for setting up variables to track the state of mechanical objects
   which needed to be displayed on screen 
--------------------------------------------------------------------------------------------------*/
static struct {
  int droptargetPos;    /* Skull Drop Target Position */
  int gunPos;			/* Gun Position  */
  int gunDir;		    /* Current Direction of Gun Movement*/
  int gunLDir;			/*Last direction of Gun Movement*/
} locals;

/*--------------------------
/ Game specific input ports
/---------------------------
   Here we define for WPCMAME which Keyboard Presses it should look for, and process!
--------------------------------------------------------------------------------------------------*/
WPC_INPUT_PORTS_START(t2,3)
  PORT_START /* 0 */
    WPCPORT_BIT(   0x0001,"Left Qualifier",  KEYCODE_LSHIFT)
    WPCPORT_BIT(   0x0002,"Right Qualifier", KEYCODE_RSHIFT)
    WPCPORT_BITIMP(0x0004,"L/R Ramp",        KEYCODE_R)
    WPCPORT_BITIMP(0x0008,"L/R Outlane",     KEYCODE_O)
    WPCPORT_BITIMP(0x0010,"L/R Loop",        KEYCODE_L)
    WPCPORT_BIT(   0x0040,"L/R Slingshot",   KEYCODE_MINUS)
    WPCPORT_BIT(   0x0080,"L/R Inlane",      KEYCODE_I)
    WPCPORT_BITIMP(0x0100,"Kickb Drop Targ. 1",     KEYCODE_W)
    WPCPORT_BITIMP(0x0200,"Kickb Drop Targ. 2",     KEYCODE_E)
    WPCPORT_BITIMP(0x0400,"Kickb Drop Targ. 3",     KEYCODE_R)
    WPCPORT_BITIMP(0x0800,"Skull Drop Target",	KEYCODE_T)
    WPCPORT_BIT(   0x1000,"Left Lock",			KEYCODE_Z)
    WPCPORT_BIT(   0x2000,"Jet 1",				KEYCODE_H)
    WPCPORT_BITIMP(0x4000,"Jet 2",				KEYCODE_J)
    WPCPORT_BIT(   0x8000,"Jet 3",				KEYCODE_K)
   	
  PORT_START /* 1 */
    WPCPORT_BIT(   0x0001,"Mid. Drop Targ. 1",	KEYCODE_X)
    WPCPORT_BIT(   0x0002,"Mid. Drop Targ. 2",	KEYCODE_C)
    WPCPORT_BIT(   0x0004,"Mid. Drop Targ. 3",	KEYCODE_V)
    WPCPORT_BIT(   0x0008,"Rollover 1'",        KEYCODE_A)
    WPCPORT_BIT(   0x0010,"Rollover 2",			KEYCODE_S)
    WPCPORT_BIT(   0x0020,"Rollover 3",			KEYCODE_D)
    WPCPORT_BIT(   0x0040,"L. Drop Targ. 1",	KEYCODE_N)
    WPCPORT_BIT(   0x0100,"L. Drop Targ. 2",	KEYCODE_M)
    WPCPORT_BIT(   0x0200,"L. Drop Targ. 3",	KEYCODE_COMMA)
    WPCPORT_BIT(   0x0400,"L. Drop Targ. 4",	KEYCODE_STOP)
    WPCPORT_BIT(   0x0800,"L. Drop Targ. 5",	KEYCODE_SLASH)
    WPCPORT_BIT(   0x1000,"Drain",				KEYCODE_Q)
//	WPCPORT_BITIMP(0x2000,"Home",				KEYCODE_F)
//	WPCPORT_BIT(   0x4000,"Mark",				KEYCODE_G)
//	WPCPORT_BIT(   0x8000,"Jet Bumper Lane", KEYCODE_Y)
WPC_INPUT_PORTS_END

/*-------------------
/ Switch definitions
/--------------------*/
#define swStart      	13
#define swTilt       	14
#define swLTrough    	15
#define swCTrough    	16
#define swRTrough    	17
#define swOuthole    	18

#define swSlamTilt   	21
#define swCoinDoor   	22
#define swTicket     	23
#define swNotUsed		24	
#define swLOut			25
#define swLIn			26
#define swRIn			27
#define swROut			28

#define swGunLoaded		31
#define swGunMark		32
#define swGunHome		33
#define swTrigger		34
/*35 Not Used*/
#define swMidLeft		36	
#define swMidCenter		37
#define swMidRight		38
	
#define swJet1			41
#define swJet2			42
#define swJet3			43
#define swLSling     	44
#define swRSling     	45
#define swTopRDT     	46
#define swMidRDT     	47
#define swBotRDT     	48

#define swLockL			51
/*52 Not Used*/
#define swEscapeL		53
#define swEscapeH		54
#define	swLockT			55
#define	swMultL			56
#define	swMultC			57
#define	swMultR			58

#define	swLRampEnt		61
#define	swLRampExit		62
#define	swRRampEnt		63
#define	swRRampExit		64
#define	swLChaseLoop	65
#define	swHChaseLoop	66
/*67 Not Used*/
/*68 Not Used*/

#define	swTarget1		71
#define	swTarget2		72
#define	swTarget3		73
#define	swTarget4		74
#define	swTarget5		75
#define	swBallPopper	76
#define	swDropTarget	77
#define swShooter  		78

/*81-88 Not Used*/

/*---------------------
/ Solenoid definitions
/----------------------*/

#define sBallPopper		1
#define sGunKicker		2
#define sOuthole      	3
#define sBallRel		4
#define sRSling      	5
#define sLSling      	6
#define sKnocker       	7
#define sKickBack      	8
#define sPlunger       	9
#define sTopLock       10
#define sGunMotor      11
#define sKnockDown	   12
#define sJet1		   13
#define sJet2          14
#define sJet3          15
#define sLeftLock      16
#define sDropTarget	   28

#define GUN_HOME    0	    /*Beginning Position*/
#define GUN_MARK    (18*5)  /* Where is Mark Switch Triggered. Roughly 1/3 way between home & end*/ 
#define GUN_END		(40*5)  /*# of Times to increment rotation until we stop and turnaround!*/
#define GUN_LEFT    0
#define GUN_RIGHT   1
#define DT_UP		0
#define DT_DOWN		1
/*---------------------
/  Ball state handling
/----------------------*/

/* ---------------------------------------------------------------------------------
   The following variables are used to refer to the state array!
   These vars *must* be in the *exact* *same* *order* as each stateDef array entry! 
   -----------------------------------------------------------------------------------------*/
enum {stRTrough=WPCSIM_FIRSTSTATE, stCTrough, stLTrough, stOuthole, stDrain,
      stShooter, stBallLane, stROut, stLOut,stRIn, stLIn,
      stLLoopUp, stLLoopUp2, stTopLock, stLLoopDn, stLLoopDn2, 
	  stRLoopUp, stRLoopUp2, stRLoopDn, stRLoopDn2, 
	  stLRampEnt, stLRamp, stLRampExit, stRRampEnt, stRRamp, stRRampExit,
	  stLockLeft, stDropTarget, stDropTarget2, stBallPopper, stGunLoaded, stGunFire,
	  stTarget1, stTarget2, stTarget3, stTarget4, stTarget5, stMultL, stJet1, stJet2, stJet3
      };

/********************************************************************************************************
   The following is a list of all possible game states..... 
   We start defining our own states at element #3, since 'Not Installed', 'Moving', and 'PF'
   are always defined the same!!

   Any state with all zeros, like the following example must be handled in the Handle Ball State function!
   {"State Name",    1,0,    0,       0, 0}, 

   Fields for each array element
   ----------------------------- 
   #1) Name to display on screen
   #2) Switch down time (minimum)
   #3) Switch to be triggered while in this state
   #4) Solenoid used to get out of state (0=leave ASAP) also: Balls can not enter state while solenoid is active
   #5) State following this one
   #6) Time between this state and next
   #7) If this solenoid is active go to altstate (below)
   #8) alternative state
   #9) State flags
	STSWKEEP:        Leave switch on (e.g. drop targets)
	STSHOOT:         Set ballspeed to plunger speed
	STIGNORESOL:     Always call statehandler regardless of solenoid state
	STNOTEXCL:       More than one ball can be in this sate
	STNOTBALL(type): Specified ball type does not affect switch
	STSPINNER        Switch is a spinner
*******************************************************************************************************/

static wpcsim_tState t2_stateDef[] = {
  {"Not Installed",    0,0,           0,        stDrain,     0,0,0,WPCSIM_STNOTEXCL},
  {"Moving"},
  {"PF",               0,0,           0,        0,           0,0,0,WPCSIM_STNOTEXCL},

  /*Line 1*/
  {"Right Trough",     1,swRTrough,   sBallRel, stShooter,  5},
  {"Center Trough",    1,swCTrough,   0,        stRTrough,   1},
  {"Left Trough",      1,swLTrough,   0,        stCTrough,   1},
  {"Outhole",          1,swOuthole,   sOuthole, stLTrough,   5},
  {"Drain",            1,0,           0,        stOuthole,   0,0,0,WPCSIM_STNOTEXCL},
  /*Line 2*/
  {"Shooter",		   1,swShooter,   sPlunger, stBallLane,  1,0,0,WPCSIM_STNOTEXCL},
  {"Ball Lane",		   1, 0,  	      0,        stTarget2,   5,0,0,WPCSIM_STNOTEXCL},
  {"Right Outlane",    1,swROut,      0,        stDrain,    20},
  {"Left Outlane",     1,swLOut,      0,        stDrain,    20, sKickBack, stFree},
  {"Right Inlane",     1,swRIn,       0,        stFree,      5},
  {"Left  Inlane",     1,swLIn,       0,        stFree,      5},
  /*Line 3*/
  {"Left Loop",        1,swEscapeL,    0,       stLLoopUp2,   1},	/* Left Loop Going Up     */
  {"Left Loop",        1,swEscapeH,    0,       stTopLock,   15},   /* Left Loop Going Up #2  */
  {"Top Lock",         1,swLockT,      sTopLock,stMultL,     25},
  {"Left Loop",        1,swEscapeL,    0,       stFree,       1},	/* Left Loop Coming Down */
  {"Left Loop",        1,swEscapeH,    0,       stLLoopDn,    1},   /* Left Loop Coming Down2  */
  /*Line 4*/
  {"Right Loop",       1,swLChaseLoop, 0,       stRLoopUp2,   1},	/* Right Loop Going Up     */
  {"Right Loop",       1,swHChaseLoop, 0,       stLLoopDn2,  10},	/* Right Loop Going Up #2  */
  {"Right Loop",       1,swLChaseLoop, 0,       stFree,       1},	/* Right Loop Coming Down  */
  {"Right Loop",       1,swHChaseLoop, 0,       stRLoopDn,   10},	/* Right Loop Coming Down2 */
  /*Line 5*/
  {"L. Ramp Enter",	   1,swLRampEnt,   0,       stLRamp,	 5},
  {"L. Ramp",		   1,swLRampExit,  0,       stLRampExit,15},	/*Here strickly to fix timing issues*/
  {"L. Ramp Exit",     1,0,        0,           stLIn,       2},
  {"R. Ramp Enter",	   1,swRRampEnt,   0,       stRRamp,	 5},
  {"R. Ramp",		   1,swRRampExit,  0,       stRRampExit,15},	/*Here strickly to fix timing issues*/
  {"R. Ramp Exit",     1,0,		   0,           stRIn,       2},
  /*Line 6*/
  {"Left Lock",        1,swLockL,  sLeftLock,stFree,      10},
  {"Skull Drop Target",1,0,  0,  0,  0},
  {"Skull Drop Target",1,swDropTarget, 0, stFree, 1, 0,0, WPCSIM_STSWKEEP},
  {"Ball Popper",	   1,swBallPopper, sBallPopper, stGunLoaded, 5}, 
  {"Gun Loaded" ,	   1,swGunLoaded,  sGunKicker, stGunFire, 1},
  {"Gun Fired",		   1,0,0,0,0},
  /*Line 7*/
  {"Orbit Target 1",   1,swTarget1, 0, stFree, 1},
  {"Orbit Target 2",   1,swTarget2, 0, stFree, 1},
  {"Orbit Target 3",   1,swTarget3, 0, stFree, 1},
  {"Orbit Target 4",   1,swTarget4, 0, stFree, 1},
  {"Orbit Target 5",   1,swTarget5, 0, stFree, 1},
  {"Left Rollover",    1,swMultL,   0, stJet1, 10},
  {"Jet Bumper 1",       1,swJet1,    0, stJet2, 2},
  {"Jet Bumper 2",       1,swJet2,    0, stJet3, 2},
  {"Jet Bumper 3",       1,swJet3,    0, stFree, 3},
  {0}
};

static int t2_handleBallState(wpcsim_tBallStatus *ball, int *inports) {
  switch (ball->state)
	{
		/*-----------------
		  Skull Drop Target 
		-------------------*/
		case stDropTarget:
		/* If Drop Target Is Down.. Shoot to Ball Popper*/
		/* Otherwise simply trigger Drop Target Switch  */
		if (locals.droptargetPos==DT_DOWN)
			return setState(stBallPopper,5);
		else
			return setState(stDropTarget2,5);
		break;

		/* Gun Fired   */
		/* - Gun Hits Targets based on Position
			 Position (Mark -> Mark + 14)  = No Target Hit!
					  (Mark+15  ->  Mark+29)  = Target 1
					  (Mark+30  ->  Mark+44)  = Target 2
					  (Mark+45  ->  Mark+59)  = Target 3
					  (Mark+60  ->  Mark+74)  = Target 4
					  (Mark+75  ->  Mark+89)  = Target 5
					  (Mark+90  ->  Mark+104) = No Target Hit!
		*/

		case stGunFire:
			if(locals.gunPos>= GUN_MARK && locals.gunPos<= GUN_MARK+14)
				return setState(stFree,2);				
			if(locals.gunPos>= GUN_MARK+15 && locals.gunPos<= GUN_MARK+29)
				return setState(stTarget1,2);
			if(locals.gunPos>= GUN_MARK+30 && locals.gunPos<= GUN_MARK+44)
				return setState(stTarget2,2);
			if(locals.gunPos>= GUN_MARK+45 && locals.gunPos<= GUN_MARK+59)
				return setState(stTarget3,2);
			if(locals.gunPos>= GUN_MARK+60 && locals.gunPos<= GUN_MARK+74)
				return setState(stTarget4,2);
			if(locals.gunPos>= GUN_MARK+75 && locals.gunPos<= GUN_MARK+89)
				return setState(stTarget5,2);
			if(locals.gunPos>= GUN_MARK+90 && locals.gunPos<= GUN_MARK+104)
				return setState(stFree,2);
		return setState(stFree,2);
		break;
  	}
  return 0;
}

/*---------------------------
/  Keyboard conversion table
/----------------------------
  Each entry maps a keypress to either begin a state or trigger a switch! 
  The Inport #, and Bit Mask define which keypresses should be used - See code from 
  WPC_INPUT_PORTS_START to determine # and Mask for each key press
-------------------------------------------------------------------------------------------------- 
  Layout of inportData array
   ----------------------------
   #1) Inport number
   #2) Mask for this switch (all bits must be set for switch to activate
       Only first match will be used. Placed qualified keys first
   #3) Switch to activate or new ball state
   #4) Condition for the action to occur
                 0                  : ball in stFree state
                 switch             : ball in stFree state and switch active
                 WPCSIM_STANY       : ball can be in any state
                 WPCSIM_CUSTCOND(n) : call custom condition handler (keyCond)
--------------------------------------------------------------------------------------------------*/

static wpcsim_tInportData t2_inportData[] = {
  {0, 0x0005, stLRampEnt},
  {0, 0x0006, stRRampEnt},
  {0, 0x0009, stLOut},
  {0, 0x000a, stROut},
  {0, 0x0011, stLLoopUp},
  {0, 0x0012, stRLoopUp},
  {0, 0x0041, swLSling},
  {0, 0x0042, swRSling},
  {0, 0x0081, stLIn},
  {0, 0x0082, stRIn},
  {0, 0x0100, swTopRDT},
  {0, 0x0200, swMidRDT},
  {0, 0x0400, swBotRDT},
  {0, 0x0800, stDropTarget},
  {0, 0x1000, stLockLeft},
  {0, 0x2000, swJet1},
  {0, 0x4000, swJet2},
  {0, 0x8000, swJet3},
  {1, 0x0001, swMidLeft},
  {1, 0x0002, swMidCenter},
  {1, 0x0004, swMidRight},
  {1, 0x0008, swMultL},
  {1, 0x0010, swMultC},
  {1, 0x0020, swMultR},
  {1, 0x0040, swTarget1},
  {1, 0x0100, swTarget2},
  {1, 0x0200, swTarget3},
  {1, 0x0400, swTarget4},
  {1, 0x0800, swTarget5},
  {1, 0x1000, stDrain},
  {1, 0x2000, swGunHome},
  {1, 0x4000, swGunMark},
  {0}
};

/*--------------------
  Drawing information
  --------------------------------------------------------
  Code to draw the mechanical objects, and their states! 
---------------------------------------------------------*/
static void t2_drawMech(unsigned char **line) {
  wpc_textOutf(30, 0,BLACK,"Skull Target: %-6s", (locals.droptargetPos==DT_UP)?"Up":"Down");
  wpc_textOutf(30, 10,BLACK,"Gun Position: %3d", locals.gunPos);
}

/*-----------------
/  ROM definitions
/------------------*/
WPC_ROMSTART(t2,l8,"t2_l8.rom",0x80000,0xc00e52e9)
WPCS_SOUNDROM222("t2_u18.l3",0x2280bdd0,
                 "t2_u15.l3",0xdad03ad1,
                 "t2_u14.l3",0x9addc9dc)
WPC_ROMEND

/*--------------
/  Game drivers
/---------------*/
WPC_GAMEDEF(t2,l8,"T2 - Terminator 2: Judgement Day L-8",1991,"Williams",wpc_mDMDS,0)

/*-----------------------
/ Simulation Definitions
/-----------------------*/
static wpcsim_tSimData t2SimData = {
  2,    				/* 2 game specific input ports */
  t2_stateDef,			/* Definition of all states */
  t2_inportData,		/* Keyboard Entries */
  { stRTrough, stCTrough, stLTrough, stDrain, stDrain, stDrain, stDrain },	/*Position where balls start.. Max 7 Balls Allowed*/
  NULL, 				/* no init */
  t2_handleBallState,	/*Function to handle ball state changes*/
  t2_handleMech,		/*Function to handle mechanical state changes*/
  FALSE, 				/* Do Not Simulate manual shooter */
  NULL  				/* no custom key conditions */
};

/*----------------------
/ Game Data Information
/----------------------*/
static wpc_tGameData t2GameData = {
  WPC_GENDMD,	/* generation */
  {0}, 			/* no serial number */
  0,			/* no custom switch column */
  0,			/* no custom lampcolumns */
  0,			/* no custom soleniods/flashers */
  FLIP_SWNO(12,11), 	/* Which switches are the flippers */
  /*Inverted Switch Definitions (See Notes Below)*/
  /*Coin    1     2     3     4     5     6     7     8     9    10   Cab.  Cust */
  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, /* No Inverted Switches!!!! */
  /*Start    Tilt    SlamTilt    CoinDoor    Shooter */
  { swStart, swTilt, swSlamTilt, swCoinDoor, swTrigger},
  NULL, 			/* no custom solenoid handler needed */
  NULL, NULL, NULL, /* switch, lamp & solenoid positions */
  t2_drawMech,		/* Function to Draw the Status of Mechanical Objects on Screen */
  &t2SimData		/* Definition of Simulator Data */
};

/*---------------
/  Game handling
/----------------*/
static void init_t2(void) {
  wpc_gameData = &t2GameData;
}

/***************************************************************************************************
 Functions here must manually adjust the state of mechanicla objects, usually checking for solenoids
 which control the state of those objects. Opening & Closing Doors, Diverter Switchs, etc..
****************************************************************************************************/
static void t2_handleMech(int *inports) {
  /* -------------------------------------
     --	Open & Close Skull Drop Target  --
     ------------------------------------- */

  /*-- if DT is up, & Kick Down Solenoid firing lower it! --*/
  if(locals.droptargetPos == DT_UP && wpc_getSol(sKnockDown))
  {
  locals.droptargetPos = DT_DOWN;
  wpc_setSw(swDropTarget,1);		/*If Drop Target Down, Switch Must be Active*/
  }

  /*-- if DT is up, & Drop Target Switch Active, lower it!! --*/
  /*-- NOTE: Here because for somereason, the sKnockDown solenoid does not fire during game play --*/
  if(locals.droptargetPos == DT_UP && wpc_getSw(swDropTarget))
	locals.droptargetPos = DT_DOWN;

  /*-- if DT is down, & Drop Target Solenoid firing raise it! --*/
  if(locals.droptargetPos == DT_DOWN && wpc_getSol(sDropTarget))
  {
	locals.droptargetPos = DT_UP;
	wpc_setSw(swDropTarget,0);		/*If Drop Target Down, Switch Must be InActive*/
  }

	/* ---------------------------------- 
     --	Rotate the Gun?              --
     ---------------------------------- */

  /*--Make sure Gun Home & Mark switches set!--*/
  /*Gun Home Switch set while it's at home!*/
  if(locals.gunPos>=GUN_HOME && locals.gunPos<=GUN_HOME+10)
	wpc_setSw(swGunHome,1);
  else
	wpc_setSw(swGunHome,0);
  /*Gun Mark Switch set unless it's at mark!*/
  if(locals.gunPos>=GUN_MARK && locals.gunPos<=GUN_MARK+10)
	wpc_setSw(swGunMark,0);
  else
	wpc_setSw(swGunMark,1);


  /*-- Is Gun Motor Turning? --*/
  if(wpc_getSol(sGunMotor))
  {
	  /*Do we need to reverse direction?*/
	  if(locals.gunLDir == GUN_LEFT && locals.gunPos>=GUN_END)
			locals.gunDir = GUN_RIGHT;

	  if(locals.gunLDir == GUN_RIGHT && locals.gunPos<=GUN_HOME)
			locals.gunDir = GUN_LEFT;
	  
	  /*Capture direction*/
	  locals.gunLDir = locals.gunDir;
	  
	  /*Change it's position*/
      locals.gunPos += (locals.gunDir==GUN_LEFT)?1:-1;
  }
}
