#include "driver.h"
#include "cpu/m6809/m6809.h"
#include "sound/2151intf.h"
#include "sound/hc55516.h"
#include "sound/dac.h"
#include "wpc.h"
#include "wpcsound.h"

/* 281000  Added volume handler (not implemented) */
/*-- bank handler --*/
static WRITE_HANDLER(wpcs_rombank_w);

/*-- sound interface handlers --*/
static WRITE_HANDLER(wpcs_latch_w);
static READ_HANDLER(wpcs_latch_r);

/*-- other memory handlers --*/
static WRITE_HANDLER(wpcs_volume_w);

/*-- sound generation --*/
static void wpcs_ym2151IRQ(int state);

/*-- local data --*/
static struct {
  int replyAvail;
} locals;

/*--------------
/  Memory maps
/---------------*/
MEMORY_READ_START(wpcs_readmem)
  { 0x0000, 0x1fff, MRA_RAM },
  { 0x2401, 0x2401, YM2151_status_port_0_r },
  { 0x3000, 0x3000, wpcs_latch_r },
  { 0x4000, 0xbfff, MRA_BANK4 },
  { 0xc000, 0xffff, MRA_ROM },
MEMORY_END

MEMORY_WRITE_START(wpcs_writemem)
  { 0x0000, 0x1fff, MWA_RAM },
  { 0x2000, 0x2000, wpcs_rombank_w },
  { 0x2400, 0x2400, YM2151_register_port_0_w },
  { 0x2401, 0x2401, YM2151_data_port_0_w },
  { 0x2800, 0x2800, DAC_0_data_w },
  { 0x2c00, 0x2c00, hc55516_0_clock_set_w },  /* these might be swapped */
  { 0x3400, 0x3400, hc55516_0_digit_clock_clear_w },
  { 0x3800, 0x3800, wpcs_volume_w }, /* Volume control */
  { 0x3c00, 0x3c00, wpcs_latch_w },
MEMORY_END

/*-------------
/ Bank handler
/--------------*/
static WRITE_HANDLER(wpcs_rombank_w) {
  int bankBase;

  /*-- save value for read --*/
  *(memory_region(WPC_MEMREG_SCPU) + 0x2000) = data;

  if ((data >= 0x70) && (data <= 0x7f))
    bankBase = data & 0x0f;
  else if ((data >= 0xb0) && (data <= 0xbf))
    bankBase = (data & 0x0f) | 0x10;
  else if ((data >= 0xd0) && (data <= 0xdf))
    bankBase = (data & 0x0f) | 0x20;
  else {
    logerror("unknown sound bank %x\n",data);
    return;
  }
  cpu_setbank(4, memory_region(WPC_MEMREG_SROM) + (bankBase<<15));
}
/*----------------------
/ Other memory handlers
/-----------------------*/
static WRITE_HANDLER(wpcs_latch_w) {
  DBGLOG(("wpcs_latch_w: %2x (PC=%4x)\n",data,cpu_get_pc()));
  soundlatch2_w(0, data);
  locals.replyAvail = TRUE;
}

static READ_HANDLER(wpcs_latch_r) {
  cpu_set_irq_line(WPC_SCPUNO, M6809_IRQ_LINE, CLEAR_LINE);
  return soundlatch_r(0);
}

static WRITE_HANDLER(wpcs_volume_w) {
  /* logerror("Volume set to %d\n",data); */
}

/*----------------
/ Sound interface
/-----------------*/
struct DACinterface wpcs_dacInt =
  { 1, { 50 }};

struct YM2151interface wpcs_ym2151Int = {
  1, 3579545, /* Hz */
  { YM3012_VOL(10,MIXER_PAN_CENTER,30,MIXER_PAN_CENTER) },
  { wpcs_ym2151IRQ }
};

struct hc55516_interface wpcs_hc55516Int =
  { 1, { 80 }};

/*-------------------
/ Exported interface
/---------------------*/
READ_HANDLER(wpcs_data_r) {
  locals.replyAvail = FALSE;
  return soundlatch2_r(0);
}

WRITE_HANDLER(wpcs_data_w) {
  #ifdef MAME_DEBUG
  char str[50];
  sprintf(str,"%4d",data);
  wpc_textOut(str,strlen(str),10,100,BLACK);
  #endif

  soundlatch_w(0, data);
  cpu_set_irq_line(WPC_SCPUNO, M6809_IRQ_LINE, ASSERT_LINE);
}

READ_HANDLER(wpcs_ctrl_r) {
  return locals.replyAvail;
}

void wpcs_init(void) {
  /* the non-paged ROM is at the end of the image. move it to its correct place */
  unsigned char *p1 = memory_region(WPC_MEMREG_SROM) + 0x080000;
  unsigned char *p2 = memory_region(WPC_MEMREG_SCPU) + 0x010000;
  int ii;

  for (ii = 0; ii < 0x4000; ii++)
    *(--p2) = *(--p1);
}

/*-----------------
/  local functions
/------------------*/
static void wpcs_ym2151IRQ(int state) {
  cpu_set_irq_line(WPC_SCPUNO, M6809_FIRQ_LINE, state ? ASSERT_LINE : CLEAR_LINE);
}



