#ifndef INC_WPCSOUND
#define INC_WPCSOUND

/* 291000 - Added SOUNDROM248 (fow WW) */
/* 161000 - Added SOUNDROM84  (for TAFG) */
/* 171000 - Added SOUNDROM8   (for FT) */
/*-- Sound rom macros --*/

#ifdef WPC_WPCSOUND
  #define WPCS_STDREG \
  ROM_REGION(0x010000, WPC_MEMREG_SCPU | REG_SOUNDONLY) \
  ROM_REGION(0x180000, WPC_MEMREG_SROM | REG_SOUNDONLY)

  #define WPCS_ROMLOAD2(start, n, chk) \
     ROM_LOAD(n, start,  0x20000, chk) \
     ROM_RELOAD( start + 0x20000, 0x20000) \
     ROM_RELOAD( start + 0x40000, 0x20000) \
     ROM_RELOAD( start + 0x60000, 0x20000)
  #define WPCS_ROMLOAD4(start, n, chk) \
     ROM_LOAD(n, start,  0x40000, chk) \
     ROM_RELOAD( start + 0x40000, 0x40000)

  #define WPCS_ROMLOAD8(start, n, chk) \
     ROM_LOAD(n, start, 0x80000, chk)

  #define WPCS_SOUNDROM882(n1,chk1,n2,chk2,n3,chk3) \
     WPCS_STDREG \
     WPCS_ROMLOAD8(0x000000, n1, chk1) \
     WPCS_ROMLOAD8(0x080000, n2, chk2) \
     WPCS_ROMLOAD2(0x100000, n3, chk3)
  #define WPCS_SOUNDROM288(n1,chk1,n2,chk2,n3,chk3) \
     WPCS_STDREG \
     WPCS_ROMLOAD2(0x000000, n1, chk1) \
     WPCS_ROMLOAD8(0x080000, n2, chk2) \
     WPCS_ROMLOAD8(0x100000, n3, chk3)
  #define WPCS_SOUNDROM222(n1,chk1,n2,chk2,n3,chk3) \
     WPCS_STDREG \
     WPCS_ROMLOAD2(0x000000, n1, chk1) \
     WPCS_ROMLOAD2(0x080000, n2, chk2) \
     WPCS_ROMLOAD2(0x100000, n3, chk3)
  #define WPCS_SOUNDROM248(n1,chk1,n2,chk2,n3,chk3) \
     WPCS_STDREG \
     WPCS_ROMLOAD2(0x000000, n1, chk1) \
     WPCS_ROMLOAD4(0x080000, n2, chk2) \
     WPCS_ROMLOAD8(0x100000, n3, chk3)
  #define WPCS_SOUNDROM84(n1,chk1,n2,chk2) \
     WPCS_STDREG \
     WPCS_ROMLOAD8(0x000000, n1, chk1) \
     WPCS_ROMLOAD4(0x080000, n2, chk2)
  #define WPCS_SOUNDROM8(n1,chk1) \
     WPCS_STDREG \
     WPCS_ROMLOAD8(0x000000, n1, chk1)

#else
  #define WPCS_SOUNDROM882(n1,chk1,n2,chk2,n3,chk3)
  #define WPCS_SOUNDROM288(n1,chk1,n2,chk2,n3,chk3)
  #define WPCS_SOUNDROM222(n1,chk1,n2,chk2,n3,chk3)
  #define WPCS_SOUNDROM248(n1,chk1,n2,chk2,n3,chk3)
  #define WPCS_SOUNDROM84(n1,chk1,n2,chk2)
  #define WPCS_SOUNDROM8(n1,chk1)
#endif /* WPC_WPCSOUND */

/*-- Machine structure externals --*/
extern const struct Memory_ReadAddress  wpcs_readmem[];
extern const struct Memory_WriteAddress wpcs_writemem[];

extern struct DACinterface      wpcs_dacInt;
extern struct YM2151interface   wpcs_ym2151Int;
extern struct hc55516_interface wpcs_hc55516Int;


/*-- Sound interface communications --*/
#ifdef WPC_WPCSOUND
  extern READ_HANDLER(wpcs_data_r);
  extern WRITE_HANDLER(wpcs_data_w);
  extern READ_HANDLER(wpcs_ctrl_r);
  extern void wpcs_init(void);
#else /* WPC_WPCSOUND */
  #define wpcs_init()       ((void*)0)
  #define wpcs_data_w(x,y)  ((void*)0)
  #define wpcs_data_r(x)    (0)
  #define wpcs_ctrl_r(x) (0)
#endif /* WPC_WPCSOUND */

/*  CPU_M6809 | CPU_SOUNDONLY, \ */
#define WPCS_SOUNDCPU ,{ \
  CPU_M6809, \
  2000000, /* 2 MHz */ \
  wpcs_readmem, wpcs_writemem, 0, 0, \
  ignore_interrupt, 0 \
}

#define WPCS_SOUND \
  { SOUND_YM2151,  &wpcs_ym2151Int }, \
  { SOUND_DAC,     &wpcs_dacInt }, \
  { SOUND_HC55516, &wpcs_hc55516Int }

#endif /* INC_WPCSOUND */
